package com.xxh.cloud.data.statistics.framework.modules.task.mapper;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.xxh.cloud.data.statistics.framework.modules.query.service.bean.bos.DeptStatBO;
import com.xxh.cloud.data.statistics.framework.modules.query.service.bean.bos.DiseaseInfoStatBO;
import com.xxh.cloud.data.statistics.framework.modules.query.service.bean.bos.PatientInfoStatBO;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.*;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;

import java.util.List;

public interface StatisticsDataQueryByBizMapper {


    /**
     * 从标签中统计,潜在患者【疾病】列表
     * */
    @Select(" SELECT a.patient_id as patientId ,a.main_patient_id as mainPatientId ," +
            "c.disease_id as diseaseId  ,max(a.data_dt) as dataTime FROM h_patient_label_history_info a " +
            " INNER JOIN s_md_standard_label_info b on (a.label_id = b.id)" +
            " INNER JOIN h_entity_disease_config_info c on (b.id = c.entity_id)  ${ew.customSqlSegment}")
    public List<PatientLabelDayQueryBO> queryPatientLabelDay(@Param(Constants.WRAPPER) Wrapper<PatientLabelDayQueryBO> wrapper);



    /**
     * 日维度，从潜在患者就诊明细中，统计疾病潜在患者数量和确诊数量
     * */
    @Select("SELECT disease_id as diseaseId,count(1) as totalNum, count(case WHEN is_diagnosis = 'Y' THEN 1 END )  as diagnosisNum " +
            "FROM sd_potential_patient_not_diagnosis_detail_info   ${ew.customSqlSegment}")
    public List<PotentialPatientNotDiagnosisDayQueryBO> queryPotentialPatientNotDiagnosisDay(@Param(Constants.WRAPPER) Wrapper<PotentialPatientNotDiagnosisDayQueryBO> wrapper);



    /**
     * 从复诊信息表中查询
     * */
    @Select(" SELECT id, patient_id, DATE_FORMAT(visit_start_time, '%Y-%m-%d') as dataTime, source_time as discovery_time, visit_start_time, visit_end_time, " +
            " dept_ids " +
            " FROM h_patient_return_visit_info ${ew.customSqlSegment} " )
    public List<PatientFollowupQueryBO> queryPatientFollowup(@Param(Constants.WRAPPER) Wrapper<PatientFollowupQueryBO> wrapper);
    
    /**
     * 日维度，从复诊患者明细中，统计应复诊和已复诊人数
     * */
    @Select("SELECT count(1) as followup_num, count(case WHEN is_followup = 'Y' THEN 1 END ) as followup_yes_num " +
            "FROM sd_patient_followup_detail_info  ${ew.customSqlSegment} ")
    public PatientFollowupDayQueryBO queryPatientFollowupDay(@Param(Constants.WRAPPER) Wrapper<PatientFollowupDayQueryBO> wrapper);

    /**
     * 日维度，住院患者日统计
     * */
    @Select("SELECT count(1) as num, count(case WHEN is_inpatient = 'Y' THEN 1 END ) as inpatient_num " +
            "FROM sd_patient_inpatient_loss_detail_info  ${ew.customSqlSegment} ")
    public PatientPatientInpatientLossQueryBO queryPatientInpatientLossDay(@Param(Constants.WRAPPER) Wrapper<PatientPatientInpatientLossQueryBO> wrapper);

    /**
     * 日维度，首发患者
     * */
    @Select("SELECT count(1) as num, disease_id " +
            "FROM sd_first_disease_patient_info  ${ew.customSqlSegment} ")
    public List<FirstDiseaseDayInfoQueryBO> queryFirstDiseasePatientDay(@Param(Constants.WRAPPER) Wrapper<FirstDiseaseDayInfoQueryBO> wrapper);


    /**
     * 查询关联ICD
     * */
    @Select(" select GROUP_CONCAT(icd10_code, ',') as icd10_codes, disease_id from s_md_standard_disease_icd_rel " +
            " ${ew.customSqlSegment}" )
    public List<DiseaseIcdRelBO> queryDiseaseIcdRelConcat(@Param(Constants.WRAPPER) Wrapper<DiseaseIcdRelBO> wrapper);

    /**
     * 查询关联ICD
     * */
    @Select(" select icd10_code, disease_id from s_md_standard_disease_icd_rel " +
            " ${ew.customSqlSegment}" )
    public List<DiseaseIcdRelBO> queryDiseaseIcdRel(@Param(Constants.WRAPPER) Wrapper<DiseaseIcdRelBO> wrapper);


    /**关联病种信息 */
    @Select(" select * from s_md_standard_disease_info " +
            " ${ew.customSqlSegment}" )
    public List<DiseaseInfoStatBO> queryDiseaseInfo(@Param(Constants.WRAPPER) Wrapper<DiseaseInfoStatBO> wrapper);

    /**关联部门信息 */
    @Select(" select * from sys_dept " +
            " ${ew.customSqlSegment}" )
    public List<DeptStatBO> queryDeptInfo(@Param(Constants.WRAPPER) Wrapper<DeptStatBO> wrapper);

    /**关联患者信息 */
    @Select(" select * from hd_task_patient_info" +
            " ${ew.customSqlSegment}" )
    public List<PatientInfoStatBO> queryPatientInfo(@Param(Constants.WRAPPER) Wrapper<PatientInfoStatBO> wrapper);

    // 更新 h_patient_visit_info
    @Update(" UPDATE h_patient_return_visit_info SET ${ew.sqlSet} ${ew.customSqlSegment} ")
    Integer updatePatientReturnVisit(@Param(Constants.WRAPPER) Wrapper<Void> wrapper);

    
    /**查询患者服务推送 */
    @Select(" SELECT patient_id patientId, patient_push_serve_id servId, push_serve_id servConfigId, op_date servTime FROM h_patient_push_serve_history_info   " +
            " ${ew.customSqlSegment}" )
    public List<PushDetailQueryBO> queryPushServeInfo(@Param(Constants.WRAPPER) Wrapper<Void> wrapper);

    /**查询患者服务推送 */
    @Select(" SELECT patient_id patientId, patient_push_serve_id servId, send_date pushTime FROM h_patient_push_serve_date_info   " +
            " ${ew.customSqlSegment}" )
    public List<PushDetailQueryBO> queryPushServeDateInfo(@Param(Constants.WRAPPER) Wrapper<Void> wrapper);

    /**查询患者服务推送提醒 */
    @Select(" SELECT id remindId, biz_id servId, user_id userId, patient_id patientId, push_date pushTime, push_status pushStatus FROM h_patient_remind_info   " +
            " ${ew.customSqlSegment}" )
    public List<PushDetailQueryBO> queryPushRemindInfo(@Param(Constants.WRAPPER) Wrapper<Void> wrapper);

    /**查询患者服务行为 */
    @Select(" SELECT source_one_id remindId, " +
            " MAX(IF(operated_type = 'browse','Y','N')) readStatus, MIN(IF(operated_type = 'browse',operated_start_date,NULL)) readTime, " +
            " MAX(IF(operated_type = 'knew' OR operated_type = 'further_reading' OR operated_type = 'register' OR operated_type = 'consultation_online' OR operated_type = 'consultation','Y','N')) actionStatus, " +
            " MIN(IF(operated_type = 'knew' OR operated_type = 'further_reading' OR operated_type = 'register' OR operated_type = 'consultation_online' OR operated_type = 'consultation' ,operated_start_date,NULL)) actionTime " +
            " FROM hb_patient_behavior_info " +
            " ${ew.customSqlSegment}" )
    public List<PushDetailQueryBO> queryPushServeBehaviorInfo(@Param(Constants.WRAPPER) Wrapper<Void> wrapper);

    /**统计患者服务推送 */
    @Select(" SELECT DATE_FORMAT(push_time, '%Y-%m-%d') as dataTime, " +
            " COUNT(1) as num, " +
            " COUNT(IF(push_status = '0',1,NULL)) pushFailUserNum, " +
            " COUNT(IF(push_status = '2',1,NULL)) pushSuccessNum, " +
            " COUNT(IF(push_status = '3',1,NULL)) pushFailNum, " +
            " COUNT(IF(push_status = '4',1,NULL)) pushInvalidNum, " +
            " COUNT(IF(push_status = '5',1,NULL)) pushFailFollowNum, " +
            " COUNT(IF(push_status = '7',1,NULL)) pushTimeoutNum, " +
            " COUNT(IF(push_status = '8',1,NULL)) pushBlackListNum, " +
            " COUNT(IF(push_status = '9',1,NULL)) pushAbnormalNum, " +
            " COUNT(IF(read_status = 'Y',1,NULL)) readNum, " +
            " COUNT(IF(action_status = 'Y',1,NULL)) actionNum " +
            " FROM sd_push_detail_info " +
            " ${ew.customSqlSegment}" )
    public List<PushDayQueryBO> queryPushDayList(@Param(Constants.WRAPPER) Wrapper<Void> wrapper);

    /**会诊申请 */
    @Select(" SELECT patient_id patientId, type sourceType, consultation_label_id lableId, IF(id IS NULL, 'N', 'Y') applyStatus, id applyId, " +
            " apply_time, IF(consultation_id IS NULL, 'N', 'Y') openStatus, consultation_id openId, submit_time openTime, " +
            " IF(finish_time IS NULL, 'N', 'Y') resultStatus, finish_time resultTime " +
            " FROM mdt_consultation_launch_apply_info " +
            " ${ew.customSqlSegment}" )
    public List<ConsultationDetailQueryBO> queryConsultationApplyDateInfo(@Param(Constants.WRAPPER) Wrapper<Void> wrapper);

    /**会诊标签 */
    @Select(" SELECT patient_id patientId, label_id lableId, op_date discoverTime FROM h_patient_label_history_info   " +
            " ${ew.customSqlSegment}" )
    public List<ConsultationDetailQueryBO> queryConsultationPatientLabelInfo(@Param(Constants.WRAPPER) Wrapper<Void> wrapper);

    /**会诊标签 */
    @Select(" SELECT b.id applyId, a.dept_id deptCode  " +
            " FROM sys_user a " +
            " INNER JOIN mdt_consultation_launch_apply_info b ON b.launch_doctor_id = a.id " +
            " ${ew.customSqlSegment}" )
    public List<ConsultationDetailQueryBO> queryDoctorDeptInfo(@Param(Constants.WRAPPER) Wrapper<Void> wrapper);

    /**发起的会诊的结论 */
    @Select(" SELECT consultation_id openId, mdt_dept_code deptCode, target_type resultTargetType, " +
            " IF(target_finish_time IS NULL, 'N', 'Y') resultTargetExeStatus, target_finish_time resultTargetExeTime " +
            " FROM mdt_his_consultation_advice_info " +
            " ${ew.customSqlSegment}" )
    public List<ConsultationDetailQueryBO> queryConsultationAdviceInfo(@Param(Constants.WRAPPER) Wrapper<Void> wrapper);

    /**统计会诊 */
    @Select(" SELECT DATE_FORMAT(discover_time, '%Y-%m-%d') as dataTime, dept_code deptCode, " +
            " COUNT(1) as discoverNum, " +
            " COUNT(IF(apply_status = 'Y',1,NULL)) applyNum, " +
            " COUNT(IF(open_status = 'Y',1,NULL)) openNum, " +
            " COUNT(IF(result_target_type = '1',1,NULL)) changeNum, " +
            " COUNT(IF(result_target_type = '2',1,NULL)) followUpNum, " +
            " COUNT(IF(result_target_type <> '1' AND result_target_type <> '2',1,NULL)) abandonNum " +
            " FROM sd_consultation_detail_info " +
            " ${ew.customSqlSegment}" )
    public List<ConsultationDayQueryBO> queryConsultationDayList(@Param(Constants.WRAPPER) Wrapper<Void> wrapper);

    /////////
    
    /**随访信息 */
    @Select(" SELECT patient_id patientId, source_dept_id deptCode, type sourceType, id sourceDataId, source_date sourceDataTime, " +
            " follow_time planFollowTime, IF(real_time IS NULL, 'N', 'Y') followExeStatus, real_time followExeTime " +
            " FROM h_doctor_follow_up_info " +
            " ${ew.customSqlSegment}" )
    public List<FollowUpDetailQueryBO> queryFollowUpDateInfo(@Param(Constants.WRAPPER) Wrapper<Void> wrapper);

    /**复诊信息 */
    @Select(" SELECT source_id sourceDataId, real_time followVisitTime, dept_id followVisitDeptCode, IF(real_time IS NULL, 'N', 'Y') followVisitStatus " +
            " FROM h_patient_return_visit_info " +
            " ${ew.customSqlSegment}" )
    public List<FollowUpDetailQueryBO> queryReturnVisitInfo(@Param(Constants.WRAPPER) Wrapper<Void> wrapper);

    /**统计随访 */
    @Select(" SELECT DATE_FORMAT(source_data_time, '%Y-%m-%d') as dataTime, dept_code deptCode, " +
            " COUNT(1) as totalNum, " +
            " COUNT(IF(follow_exe_status = 'Y',1,NULL)) exeYesNum, " +
            " COUNT(IF(follow_exe_status <> 'Y',1,NULL)) exeNoNum, " +
            " COUNT(IF(follow_visit_status = 'Y',1,NULL)) visitNum " +
            " FROM sd_follow_up_detail_info " +
            " ${ew.customSqlSegment}" )
    public List<FollowUpDayQueryBO> queryFollowUpDayList(@Param(Constants.WRAPPER) Wrapper<Void> wrapper);

    /**关联部门信息 */
    @Select(" select biz_dept_id from s_md_standard_dept_biz_rel " +
            " ${ew.customSqlSegment}" )
    public List<String> queryDeptInfoByStandard(@Param(Constants.WRAPPER) Wrapper<Void> wrapper);

}
