package com.xxh.cloud.data.statistics.framework.modules.task.schedule.everyday;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.data.statistics.framework.modules.task.service.DiagnosisPatientDayTaskService;
import com.xxh.cloud.data.statistics.framework.modules.task.service.FirstDiseaseDayTaskService;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.web.common.task.constant.TaskModuleEnum;
import com.xxh.cloud.web.common.task.handler.EveryDayExecute;
import com.xxh.cloud.web.common.task.model.TaskJobParamModel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import java.util.List;

@Slf4j
@Component
@Async("taskExecutor")
public class FirstDiseaseDayInfoTaskEveryDay implements EveryDayExecute {

    @Autowired
    private FirstDiseaseDayTaskService firstDiseaseDayTaskService;

    /**
     * 每天统计-首发疾病数量
     * */
    @Override
    public void execute(TaskJobParamModel params) {
        if (TaskModuleEnum.firstDisease.getCode().equals(params.getModule())
                || TaskModuleEnum.AllTask.getCode().equals(params.getModule())) {
            log.info("首发疾病日统计开始 >>>>>> ");
            if (StrUtil.isNotEmpty(params.getParams())) {
                List<String> paramsList = StrJoinSplitUtils.split2List(params.getParams());
                String startDate = paramsList.get(0);
                String endDate = paramsList.get(1);
                firstDiseaseDayTaskService.syncFirstDisease(startDate, endDate);
            } else {
                String date = DateUtil.formatDate(DateUtil.offsetDay(DateUtil.date(), -1));
                firstDiseaseDayTaskService.syncFirstDisease(date);
            }
            log.info("首发疾病日统计结束 <<<<<< ");
        }

    }
}
