package com.xxh.cloud.data.statistics.framework.modules.task.schedule.everyday;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.data.statistics.framework.modules.task.service.DiagnosisPatientDayTaskService;
import com.xxh.cloud.data.statistics.framework.modules.task.service.PatientFollowupDayTaskService;
import com.xxh.cloud.web.common.task.constant.TaskModuleEnum;
import com.xxh.cloud.web.common.task.handler.EveryDayExecute;
import com.xxh.cloud.web.common.task.model.TaskJobParamModel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

@Slf4j
@Component
@Async("taskExecutor")
public class PatientFollowupDayTaskEveryDay implements EveryDayExecute {

    @Autowired
    private PatientFollowupDayTaskService patientFollowupDayTaskService;

    /**
     * 每天统计前一天的复诊记录
     * */
    @Override
    public void execute(TaskJobParamModel params) {
        if (TaskModuleEnum.patientFollowup.getCode().equals(params.getModule())
                || TaskModuleEnum.AllTask.getCode().equals(params.getModule())) {
            log.info("复诊患者日统计开始 >>>>>> ");
            String date = DateUtil.formatDate(DateUtil.offsetDay(DateUtil.date(), -1));
            if (StrUtil.isNotEmpty(params.getParams())) {
                date = params.getParams();
            }

            // 按时间同步复诊患者
            patientFollowupDayTaskService.syncFollowupPatient(date);
            // 计算复诊患者是否就诊
            patientFollowupDayTaskService.calculateFollowup(date);

            log.info("复诊患者日统计结束 <<<<<< ");
        }

    }
}
