package com.xxh.cloud.data.statistics.framework.modules.task.schedule.everyday;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.data.statistics.framework.modules.task.service.PatientFollowupDayTaskService;
import com.xxh.cloud.data.statistics.framework.modules.task.service.PatientInpatientLossDayTaskService;
import com.xxh.cloud.web.common.task.constant.TaskModuleEnum;
import com.xxh.cloud.web.common.task.handler.EveryDayExecute;
import com.xxh.cloud.web.common.task.model.TaskJobParamModel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

@Slf4j
@Component
@Async("taskExecutor")
public class PatientInpatientLossDayTaskEveryDay implements EveryDayExecute {

    @Autowired
    private PatientInpatientLossDayTaskService patientInpatientLossDayTaskService;

    /**
     * 每天统计前一天的住院流失记录
     * */
    @Override
    public void execute(TaskJobParamModel params) {
        if (TaskModuleEnum.patientInpatientLoss.getCode().equals(params.getModule())
                || TaskModuleEnum.AllTask.getCode().equals(params.getModule())) {
            log.info("住院流失记录日统计开始 >>>>>> ");
            String date = DateUtil.formatDate(DateUtil.offsetDay(DateUtil.date(), -1));
            if (StrUtil.isNotEmpty(params.getParams())) {
                date = params.getParams();
            }

            // 按时间同步开住院单患者
            patientInpatientLossDayTaskService.syncInpatientApply(date);
            // 计算开住院单患者是否住院
            patientInpatientLossDayTaskService.calculateInpatientApply(date);

            log.info("住院流失记录日统计结束 <<<<<< ");
        }

    }
}
