package com.xxh.cloud.data.statistics.framework.modules.task.schedule.everyday;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.data.statistics.framework.modules.task.service.PotentialPatientNotDiagnosisDayTaskService;
import com.xxh.cloud.web.common.task.constant.TaskModuleEnum;
import com.xxh.cloud.web.common.task.handler.EveryDayExecute;
import com.xxh.cloud.web.common.task.model.TaskJobParamModel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

@Slf4j
@Component
@Async("taskExecutor")
public class PotentialPatientNotDiagnosisDayInfoTaskEveryDay implements EveryDayExecute {

    @Autowired
    private PotentialPatientNotDiagnosisDayTaskService potentialPatientNotDiagnosisDayTaskService;

    /**
     * 每天统计-前一日的潜在患者未就诊日统计
     * */
    @Override
    public void execute(TaskJobParamModel params) {
        if (TaskModuleEnum.potentialPatient.getCode().equals(params.getModule())
                || TaskModuleEnum.AllTask.getCode().equals(params.getModule())) {
            log.info("潜在患者未就诊日统计开始 >>>>>> ");
            String date = DateUtil.formatDate(DateUtil.offsetDay(DateUtil.date(), -1));
            if (StrUtil.isNotEmpty(params.getParams())) {
                date = params.getParams();
            }
            // 同步潜在患者
            potentialPatientNotDiagnosisDayTaskService.syncPotentialPatient(date);
            // 计算患者是否就诊
            potentialPatientNotDiagnosisDayTaskService.calculateDiagnosis(date);
            log.info("潜在患者未就诊日统计结束 <<<<<< ");
        }

    }
}
