package com.xxh.cloud.data.statistics.framework.modules.task.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.ConsultationDayInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.ConsultationDetailInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.service.ConsultationDayInfoService;
import com.xxh.cloud.data.statistics.framework.modules.base.service.ConsultationDetailInfoService;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.ConsultationDayQueryBO;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.ConsultationDetailQueryBO;
import com.xxh.cloud.data.statistics.framework.modules.task.mapper.StatisticsDataQueryByBizMapper;
import com.xxh.cloud.data.statistics.framework.modules.task.service.ConsultationServeTaskService;
import com.xxh.cloud.framework.common.constant.YN;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class ConsultationServeTaskServiceImpl implements ConsultationServeTaskService {

    @Autowired
    private StatisticsDataQueryByBizMapper statisticsDataQueryByBizMapper;
    @Autowired
    private ConsultationDetailInfoService consultationDetailInfoService;
    @Autowired
    private ConsultationDayInfoService consultationDayInfoService;
 
    @Value("${consultation.maxDays:7}")
    private Integer maxDays; // 以会诊申请时间为基准

    @Override
    public void consultationDetailStatisticsByDate(String startDate, String endDate) {
        DateTime now = DateUtil.date();
        DateTime start = DateUtil.parseDate(startDate);
        DateTime finishDate = DateUtil.offsetDay(now, 1);
        if (StrUtil.isNotBlank(endDate)) {
            finishDate = DateUtil.parseDate(endDate);
        }

        // 循环每天处理
        while (true) {
            DateTime end = DateUtil.offsetDay(start, 1);
            if (end.after(finishDate)) {
                return;
            }
            String status = YN.YES.getCode();
            if (DateUtil.offsetDay(start, maxDays).after(now)) { // 若最大数据变化日期大于当前日期则说明该数据需要后续更新
                status = YN.NO.getCode();
            }
            consultationDetailStatisticsByDateHander(start.toDateStr(), end.toDateStr(), status);
            start = end;
        }
    }

    public void consultationDetailStatisticsByDateHander(String startDate, String endDate, String status) {
        if (StrUtil.isBlank(startDate)) {
            return;
        }

        // 1.根据会诊标签查询会诊申请信息
        List<ConsultationDetailQueryBO> consultationApplyList = statisticsDataQueryByBizMapper.queryConsultationApplyDateInfo(new QueryWrapper<Void>()
            .ge("apply_time", startDate)
            .lt(StrUtil.isNotBlank(endDate), "apply_time", endDate)
        );

        if (CollUtil.isEmpty(consultationApplyList)) {
            return;
        }

        Set<String> applyIds = new HashSet<>();
        Set<String> consultationPatientLabelIds = new HashSet<>();
        Set<String> consultationIds = new HashSet<>();
        consultationApplyList.forEach(i -> {
            applyIds.add(i.getApplyId());
            consultationPatientLabelIds.add(i.getPatientId() + "-" + i.getLableId());
            if (StrUtil.isNotBlank(i.getOpenId())) {
                consultationIds.add(i.getOpenId());
            }
        });

        // 2.查询会诊标签患者信息
        Map<String, ConsultationDetailQueryBO> consultationPatientLabelMap = new HashMap<>();
        if (CollUtil.isNotEmpty(consultationPatientLabelIds)) {
            List<ConsultationDetailQueryBO> consultationPatientLabelList = statisticsDataQueryByBizMapper.queryConsultationPatientLabelInfo(new QueryWrapper<Void>()
                .eq("op_type", "1")
                .in("CONCAT(patient_id, '-', label_id)", consultationPatientLabelIds)
            );
            if (CollUtil.isNotEmpty(consultationPatientLabelList)) {
                consultationPatientLabelList.forEach(i -> consultationPatientLabelMap.put(i.getPatientId() + "-" + i.getLableId(), i));
            }
        }

        // 3.根据会诊申请id查询会诊医生
        Map<String, ConsultationDetailQueryBO> doctorDeptMap = new HashMap<>();
        if (CollUtil.isNotEmpty(applyIds)) {
            List<ConsultationDetailQueryBO> doctorDeptList = statisticsDataQueryByBizMapper.queryDoctorDeptInfo(new QueryWrapper<Void>()
                .in("b.id", applyIds)
            );
            if (CollUtil.isNotEmpty(doctorDeptList)) {
                doctorDeptList.forEach(i -> doctorDeptMap.put(i.getApplyId(), i));
            }
        }

        // 4.根据会诊id查询会诊结论，有会诊结论说明会诊完成，会诊结论上有会诊目标及目标完成时间（转科、随访、放弃）
        Map<String, ConsultationDetailQueryBO> consultationAdviceMap = new HashMap<>();
        if (CollUtil.isNotEmpty(consultationIds)) {
            List<ConsultationDetailQueryBO> consultationAdviceList = statisticsDataQueryByBizMapper.queryConsultationAdviceInfo(new QueryWrapper<Void>()
                .in("consultation_id", consultationIds)
            );
            if (CollUtil.isNotEmpty(consultationAdviceList)) {
                consultationAdviceList.forEach(i -> consultationAdviceMap.put(i.getOpenId(), i));
            }
        }

        // 5.数据整理
        List<ConsultationDetailQueryBO> boList = new ArrayList<>();
        consultationApplyList.forEach(i -> {
            i.setStatus(status);

            ConsultationDetailQueryBO consultationPateintLabel = consultationPatientLabelMap.get(i.getPatientId() + "-" + i.getLableId());
            if (null != consultationPateintLabel) {
                i.setDiscoverTime(consultationPateintLabel.getDiscoverTime());
            }

            ConsultationDetailQueryBO doctorDept = doctorDeptMap.get(i.getApplyId());
            if (null != doctorDept) {
                i.setDeptCode(doctorDept.getDeptCode());
            }

            ConsultationDetailQueryBO consultationAdvice = consultationAdviceMap.get(i.getOpenId());
            if (null != consultationAdvice) {
                i.setResultTargetType(consultationAdvice.getResultTargetType());
                i.setResultTargetExeStatus(consultationAdvice.getResultTargetExeStatus());
                i.setResultTargetExeTime(consultationAdvice.getResultTargetExeTime());

                if (null != i.getResultTargetExeTime()) {
                    i.setStatus(YN.YES.getCode());
                }
            }

            boList.add(i);
        });
        
        // 数据是否存在
        Map<String, ConsultationDetailInfo> existMap = new HashMap<>();
        if (CollUtil.isNotEmpty(boList)) {
            List<String> existParams = new ArrayList<>();
            boList.forEach(i -> {
                existParams.add(i.getApplyId());
            });
            if (CollUtil.isNotEmpty(existParams)) {
                List<ConsultationDetailInfo> existList = consultationDetailInfoService.list(new QueryWrapper<ConsultationDetailInfo>()
                    .in("apply_id", existParams)
                );
                if (CollUtil.isNotEmpty(existList)) {
                    existList.forEach(i -> {
                        existMap.put(i.getApplyId(), i);
                    });
                }
            }
        }

        Date now = DateUtil.date();
        // 保存
        List<ConsultationDetailInfo> saveList = new ArrayList<>();
        List<ConsultationDetailInfo> updateList = new ArrayList<>();
        if (CollUtil.isNotEmpty(boList)) {
            boList.forEach(i -> {
                ConsultationDetailInfo info = existMap.get(i.getApplyId());
                if (null == info) {
                    info = new ConsultationDetailInfo();
                    info.setId(IdUtil.getSnowflake(1,1).nextIdStr());
                    saveList.add(info);
                } else {
                    updateList.add(info);
                }
                info.setDeptCode(i.getDeptCode());
                info.setPatientId(i.getPatientId());
                info.setSourceType(i.getSourceType());
                info.setLableId(i.getLableId());
                info.setDiscoverTime(i.getDiscoverTime());
                info.setApplyStatus(i.getApplyStatus());
                info.setApplyId(i.getApplyId());
                info.setApplyTime(i.getApplyTime());
                info.setOpenStatus(i.getOpenStatus());
                info.setOpenId(i.getOpenId());
                info.setOpenTime(i.getOpenTime());
                info.setResultStatus(i.getResultStatus());
                info.setResultTime(i.getResultTime());
                info.setResultTargetType(i.getResultTargetType());
                info.setResultTargetExeStatus(i.getResultTargetExeStatus());
                info.setResultTargetExeTime(i.getResultTargetExeTime());
                info.setStatus(i.getStatus());
                info.setLatestTime(now);
            });
        }

        if (CollUtil.isNotEmpty(saveList)) {
            consultationDetailInfoService.saveBatch(saveList);
        }
        if (CollUtil.isNotEmpty(updateList)) {
            consultationDetailInfoService.updateBatchById(updateList);
        }
    }

    @Override
    public void consultationDetailUpdateStatisticsByDate(String startDate, String endDate) {
        DateTime now = DateUtil.parseDate(DateUtil.formatDate(DateUtil.date()));
        
        if (StrUtil.isBlank(startDate)) {
            startDate = DateUtil.offsetDay(now, -(maxDays + 1)).toDateStr();
        }
        DateTime start = DateUtil.parseDate(startDate);
        DateTime finishDate = DateUtil.offsetDay(now, 1);
        if (StrUtil.isNotBlank(endDate)) {
            finishDate = DateUtil.parseDate(endDate);
        }
        while (true) {
            if (start.after(finishDate)) {
                return;
            }
            DateTime end = DateUtil.offsetDay(start, 1);
            
            String status = YN.YES.getCode();
            if (DateUtil.offsetDay(start, maxDays).after(now)) { // 若最大数据变化日期大于当前日期则说明该数据需要后续更新
                status = YN.NO.getCode();
            }
            consultationDetailUpdateStatisticsByDateHander(start.toDateStr(), end.toDateStr(), status);
            start = end;
        }
    }
    
    public void consultationDetailUpdateStatisticsByDateHander(String startDate, String endDate, String status) {
        // 按指定配置的时间查询
        List<ConsultationDetailInfo> list = consultationDetailInfoService.list(new QueryWrapper<ConsultationDetailInfo>()
            .eq("status", YN.NO.getCode())
            .ge("apply_time", startDate)
            .lt("apply_time", endDate)
        );

        if (CollUtil.isEmpty(list)) {
            return;
        }

        // 1.会诊申请数据
        Set<String> applyIds = new HashSet<>();
        list.forEach(i -> {
            if (StrUtil.isNotBlank(i.getApplyId())) {
                applyIds.add(i.getApplyId());
            }
        });

        // DateTime maxDayDate = DateUtil.offsetDay(DateUtil.parseDate(startDate), maxDays);
        
        // 会诊申请与会诊结论
        Map<String, ConsultationDetailQueryBO> consultationApplyMap = new HashMap<>();
        Map<String, ConsultationDetailQueryBO> consultationAdviceMap = new HashMap<>();
        if (CollUtil.isNotEmpty(applyIds)) {
            List<ConsultationDetailQueryBO> consultationApplyList = statisticsDataQueryByBizMapper.queryConsultationApplyDateInfo(new QueryWrapper<Void>()
                .in("id", applyIds)
                // .ge("apply_time", startDate)
                // .lt("apply_time", maxDayDate)
            );
            if (CollUtil.isNotEmpty(consultationApplyList)) {
                
                Set<String> consultationIds = new HashSet<>();

                consultationApplyList.forEach(i -> {
                    consultationApplyMap.put(i.getApplyId(), i);
                    consultationIds.add(i.getOpenId());
                });
                
                // 会诊结论
                if (CollUtil.isNotEmpty(consultationIds)) {
                    List<ConsultationDetailQueryBO> consultationAdviceList = statisticsDataQueryByBizMapper.queryConsultationAdviceInfo(new QueryWrapper<Void>()
                        .in("consultation_id", consultationIds)
                    );
                    if (CollUtil.isNotEmpty(consultationAdviceList)) {
                        consultationAdviceList.forEach(i -> consultationAdviceMap.put(i.getOpenId(), i));
                    }
                }
            }
        }

        Date now = DateUtil.date();
        // 更新
        list.forEach(i -> {
            i.setStatus(status);
            
            ConsultationDetailQueryBO apply = consultationApplyMap.get(i.getApplyId());
            if (null != apply) {
                if (StrUtil.isNotBlank(apply.getOpenId())) {
                    i.setOpenStatus(YN.YES.getCode());
                    i.setOpenId(apply.getOpenId());
                    i.setOpenTime(apply.getOpenTime());
                }
                if (null != apply.getResultTime()) {
                    i.setResultStatus(YN.YES.getCode());
                    i.setResultTime(apply.getResultTime());
                }

                ConsultationDetailQueryBO advice = consultationAdviceMap.get(i.getOpenId());
                if (null != advice) {
                    i.setResultTargetType(advice.getResultTargetType());
                    i.setResultTargetExeStatus(advice.getResultTargetExeStatus());
                    i.setResultTargetExeTime(advice.getResultTargetExeTime());
                }
            }
            
            if (null != i.getResultTargetExeTime()) {
                i.setStatus(YN.YES.getCode());
            }

            i.setLatestTime(now);
        });

        consultationDetailInfoService.updateBatchById(list);
    }

    @Override
    public void consultationDayStatisticsByDate(String startDate, String endDate) {
        DateTime now = DateUtil.date();
        
        if (StrUtil.isBlank(startDate)) {
            startDate = DateUtil.offsetDay(now, -maxDays).toDateStr();
        }
        DateTime start = DateUtil.parseDate(startDate);
        DateTime finishDate = DateUtil.offsetDay(now, 1);
        if (StrUtil.isNotBlank(endDate)) {
            finishDate = DateUtil.parseDate(endDate);
        }

        // 循环每天处理
        while (true) {
            if (start.after(finishDate)) {
                return;
            }
            DateTime end = DateUtil.offsetDay(start, 1);
            
            consultationDayStatisticsByDateHander(start.toDateStr(), end.toDateStr());
            start = end;
        }
    }

    public void consultationDayStatisticsByDateHander(String startDate, String endDate) {
        if (StrUtil.isBlank(startDate)) {
            return;
        }
        
        List<ConsultationDayQueryBO> list = statisticsDataQueryByBizMapper.queryConsultationDayList(new QueryWrapper<Void>()
            .ge("discover_time", startDate)
            .lt(StrUtil.isNotBlank(endDate), "discover_time", endDate)
            .isNotNull("dept_code")
            .groupBy("deptCode", "dataTime")
        );
        if(CollUtil.isEmpty(list)){
            return;
        }

        Set<String> dateList = new HashSet<>();
        list.forEach(item -> {
            dateList.add(item.getDataTime());
        });

        // 查询已统计数据
        Map<String, ConsultationDayInfo> existMap = new HashMap<>();
        List<ConsultationDayInfo> existList = consultationDayInfoService.list(new QueryWrapper<ConsultationDayInfo>()
            .in("data_time", dateList)
        );
        if (CollUtil.isNotEmpty(existList)) {
            existList.forEach(i -> existMap.put(i.getDataTime(), i));
        }

        Date latestTime = new Date();

        List<ConsultationDayInfo> saveList = new ArrayList<>();
        List<ConsultationDayInfo> updateList = new ArrayList<>();

        list.forEach(item->{
            ConsultationDayInfo entity = existMap.get(item.getDataTime());
            if (null == entity) {
                entity = new ConsultationDayInfo();
                entity.setId(item.getDeptCode() + "_" + item.getDataTime());
                saveList.add(entity);
            } else {
                updateList.add(entity);
            }
            entity.setDataTime(item.getDataTime());
            entity.setDeptCode(item.getDeptCode());
            entity.setDiscoverNum(item.getDiscoverNum());
            entity.setApplyNum(item.getApplyNum());
            entity.setOpenNum(item.getOpenNum());
            entity.setChangeNum(item.getChangeNum());
            entity.setFollowUpNum(item.getFollowUpNum());
            entity.setAbandonNum(item.getAbandonNum());
            entity.setLatestTime(latestTime);

        });

        if (CollUtil.isNotEmpty(saveList)) {
            consultationDayInfoService.saveBatch(saveList);
        }
        if (CollUtil.isNotEmpty(updateList)) {
            consultationDayInfoService.updateBatchById(updateList);
        }    
    }
    
}
