package com.xxh.cloud.data.statistics.framework.modules.task.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.FirstDiseaseDayInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.FirstDiseasePatientInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.PatientFollowupDayInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.PatientInpatientLossDetailInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.service.FirstDiseaseDayInfoService;
import com.xxh.cloud.data.statistics.framework.modules.base.service.FirstDiseasePatientInfoService;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.*;
import com.xxh.cloud.data.statistics.framework.modules.task.mapper.StatisticsDataQueryByBizMapper;
import com.xxh.cloud.data.statistics.framework.modules.task.service.FirstDiseaseDayTaskService;
import com.xxh.cloud.data.statistics.framework.modules.task.service.StatisticsDwdDataQueryService;
import com.xxh.cloud.data.statistics.framework.modules.task.utils.DataStatisticsEntityIdUtils;
import com.xxh.cloud.data.statistics.framework.modules.task.utils.DataStatisticsTimeUtils;
import com.xxh.cloud.framework.common.constant.YN;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class FirstDiseaseDayTaskServiceImpl implements FirstDiseaseDayTaskService {

    @Autowired
    private FirstDiseaseDayInfoService firstDiseaseDayInfoService;
    @Autowired
    private FirstDiseasePatientInfoService firstDiseasePatientInfoService;
    @Autowired
    private StatisticsDwdDataQueryService statisticsDwdDataQueryService;
    @Resource
    private StatisticsDataQueryByBizMapper statisticsDataQueryByBizMapper;

    private String getMapKey(String patientId,String diseaseId){
        return StrUtil.format("{}:{}",patientId,diseaseId);
    }

    private FirstDiseasePatientInfo getEntity(FirstDiseaseDiagnosisQueryBO item){
        FirstDiseasePatientInfo entity = new FirstDiseasePatientInfo();
        entity.setId(IdUtil.getSnowflake(2,1).nextIdStr());
        entity.setPatientId(item.getPatientId());
        entity.setDiseaseId(item.getDiseaseId());
        entity.setDiscoveryTime(item.getDiscoveryTime());
        entity.setLatestTime(new Date());
        return entity;
    }

    private Set<String> onSync(String startDate, String endDate){
        Set<String> statisticsDates = new HashSet<>();
        List<FirstDiseaseDiagnosisQueryBO> dataList= statisticsDwdDataQueryService.getFirstDiagnosisList(startDate,endDate);
        if(CollUtil.isEmpty(dataList)){
            return statisticsDates;
        }
        Set<String> patientIdSet = new HashSet<>();
        Set<String> diseaseIdSet = new HashSet<>();
        dataList.forEach(item->{
            patientIdSet.add(item.getPatientId());
            if (StrUtil.isNotEmpty(item.getDiagnosisCode())) {
                diseaseIdSet.add(item.getDiagnosisCode());
            }
        });

        // 病种ICD关联关系
        List<DiseaseIcdRelBO> diseaseIcdRel = statisticsDataQueryByBizMapper.queryDiseaseIcdRel(new QueryWrapper<DiseaseIcdRelBO>()
                .eq("rows_status", YN.YES.getCode())
                .in("icd10_code",diseaseIdSet)
        );
        if (CollUtil.isEmpty(diseaseIcdRel)) {
            return statisticsDates;
        }
        Map<String, String> diseaseIcdMap = new HashMap<>();
        // TODO mj-li 测试代码
//        diseaseIcdMap.putAll(diseaseIcdRel.stream().collect(Collectors.toMap(DiseaseIcdRelBO::getIcd10Code, DiseaseIcdRelBO::getDiseaseId)));
        for (DiseaseIcdRelBO bo : diseaseIcdRel) {
            if (diseaseIcdMap.containsKey(bo.getIcd10Code())) {
                System.out.println(StrUtil.format("重复的icd10_{}:{}",bo.getIcd10Code(),diseaseIcdMap.get(bo.getIcd10Code())));
                System.out.println(StrUtil.format("重复的icd10_{}:{}",bo.getIcd10Code(),bo.getDiseaseId()));
            }
            diseaseIcdMap.put(bo.getIcd10Code(),bo.getDiseaseId());
        }

        // 查询患者相关的数据
        List<FirstDiseasePatientInfo> firstDiseaseList= firstDiseasePatientInfoService.list(
                new QueryWrapper<FirstDiseasePatientInfo>()
                        .in("patient_id",patientIdSet)
        );
        Map<String,FirstDiseasePatientInfo> firstDiseaseMap=new HashMap<>();
        if(CollUtil.isNotEmpty(firstDiseaseList)){
            firstDiseaseList.forEach(item->{
                firstDiseaseMap.put(getMapKey(item.getPatientId(),item.getDiseaseId()),item);
            });
        }

        List<FirstDiseasePatientInfo> saveFirstDiseaseList=new ArrayList<>();
        List<FirstDiseasePatientInfo> updateFirstDiseaseList=new ArrayList<>();

        for(FirstDiseaseDiagnosisQueryBO dataItem : dataList){
            if (!diseaseIcdMap.containsKey(dataItem.getDiagnosisCode())) {
                continue;
            }
            dataItem.setDiseaseId(diseaseIcdMap.get(dataItem.getDiagnosisCode()));
            String firstDiseaseKey = getMapKey(dataItem.getPatientId(),dataItem.getDiseaseId());

            if (firstDiseaseMap.containsKey(firstDiseaseKey)) {
                // 判断时间是否需要更新
                FirstDiseasePatientInfo entity = firstDiseaseMap.get(firstDiseaseKey);
                if (entity.getDiscoveryTime().compareTo(dataItem.getDiscoveryTime()) > 0) {
                    statisticsDates.add(DateUtil.formatDate(entity.getDiscoveryTime()));
                    statisticsDates.add(DateUtil.formatDate(dataItem.getDiscoveryTime()));
                    entity.setDiscoveryTime(dataItem.getDiscoveryTime());
                    entity.setLatestTime(new Date());
                    updateFirstDiseaseList.add(entity);
                }
            } else {
                // 首次添加
                saveFirstDiseaseList.add(this.getEntity(dataItem));
                statisticsDates.add(DateUtil.formatDate(dataItem.getDiscoveryTime()));
            }
        }
        if (CollUtil.isNotEmpty(saveFirstDiseaseList)) {
            firstDiseasePatientInfoService.saveBatch(saveFirstDiseaseList);
        }
        if (CollUtil.isNotEmpty(updateFirstDiseaseList)) {
            firstDiseasePatientInfoService.updateBatchById(updateFirstDiseaseList);
        }

        return statisticsDates;
    }

    /**
     *
     * */
    private void statistics(String date){
        List<FirstDiseaseDayInfoQueryBO> list = statisticsDataQueryByBizMapper.queryFirstDiseasePatientDay(
                new QueryWrapper<FirstDiseaseDayInfoQueryBO>()
                        .eq("DATE_FORMAT(discovery_time, '%Y-%m-%d')",date)
                        .groupBy("disease_id")
        );

        if(CollUtil.isEmpty(list)){
            return;
        }
        List<FirstDiseaseDayInfo> saveEntityList = new ArrayList<>();
        list.forEach(bo->{
            FirstDiseaseDayInfo entity = new FirstDiseaseDayInfo();
            entity.setId(IdUtil.getSnowflake(2,1).nextIdStr());
            entity.setDataTime(date);
            entity.setDiseaseId(bo.getDiseaseId());
            entity.setNum(bo.getNum());
            entity.setLatestTime(new Date());
            saveEntityList.add(entity);
        });

        firstDiseaseDayInfoService.remove(new QueryWrapper<FirstDiseaseDayInfo>()
                .eq("data_time",date)
        );
        if (CollUtil.isNotEmpty(saveEntityList)) {
            firstDiseaseDayInfoService.saveBatch(saveEntityList);
        }
    }


    @Override
    public void syncFirstDisease(String date) {
        Date date1= DataStatisticsTimeUtils.str2Date(date);
        String begin= DataStatisticsTimeUtils.beginOfDay(date1);
        String end= DataStatisticsTimeUtils.endOfDay(date1);
        this.syncFirstDisease(begin, end);
    }


    @Override
    public void syncFirstDisease(String startDate, String endDate) {
        Set<String> statisticsDates = this.onSync(startDate, endDate);
        if (CollUtil.isNotEmpty(statisticsDates)) {
            statisticsDates.forEach(this::statistics);
        }
    }



}
