package com.xxh.cloud.data.statistics.framework.modules.task.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.FollowUpDayInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.FollowUpDetailInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.service.FollowUpDayInfoService;
import com.xxh.cloud.data.statistics.framework.modules.base.service.FollowUpDetailInfoService;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.FollowUpDayQueryBO;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.FollowUpDetailQueryBO;
import com.xxh.cloud.data.statistics.framework.modules.task.mapper.StatisticsDataQueryByBizMapper;
import com.xxh.cloud.data.statistics.framework.modules.task.service.FollowUpServeTaskService;
import com.xxh.cloud.framework.common.constant.YN;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class FollowUpServeTaskServiceImpl implements FollowUpServeTaskService {

    @Autowired
    private StatisticsDataQueryByBizMapper statisticsDataQueryByBizMapper;
    @Autowired
    private FollowUpDetailInfoService followUpDetailInfoService;
    @Autowired
    private FollowUpDayInfoService followUpDayInfoService;
 
    @Value("${followUp.maxDays:7}")
    private Integer maxDays; // 以随访来源时间为基准进行新增，以随访计划时间进行更新

    @Override
    public void followUpDetailStatisticsByDate(String startDate, String endDate) {
        DateTime now = DateUtil.date();
        DateTime start = DateUtil.parseDate(startDate);
        DateTime finishDate = DateUtil.offsetDay(now, 1);
        if (StrUtil.isNotBlank(endDate)) {
            finishDate = DateUtil.parseDate(endDate);
        }

        // 循环每天处理
        while (true) {
            DateTime end = DateUtil.offsetDay(start, 1);
            if (end.after(finishDate)) {
                return;
            }
            
            followUpDetailStatisticsByDateHander(start.toDateStr(), end.toDateStr());
            start = end;
        }
    }

    public void followUpDetailStatisticsByDateHander(String startDate, String endDate) {
        if (StrUtil.isBlank(startDate)) {
            return;
        }

        // 1.随访信息
        List<FollowUpDetailQueryBO> followUpList = statisticsDataQueryByBizMapper.queryFollowUpDateInfo(new QueryWrapper<Void>()
            .ge("source_date", startDate) // 来源时间
            .lt(StrUtil.isNotBlank(endDate), "source_date", endDate)
        );

        if (CollUtil.isEmpty(followUpList)) {
            return;
        }

        Set<String> followUpIds = new HashSet<>();
        followUpList.forEach(i -> {
            followUpIds.add(i.getId());
        });

        // 2.复诊信息
        Map<String, FollowUpDetailQueryBO> returnVisitMap = new HashMap<>();
        if (CollUtil.isNotEmpty(followUpIds)) {
            List<FollowUpDetailQueryBO> returnVisitList = statisticsDataQueryByBizMapper.queryReturnVisitInfo(new QueryWrapper<Void>()
                .eq("source_type", "follow_up")
                .in("source_id", followUpIds)
            );
            if (CollUtil.isNotEmpty(returnVisitList)) {
                returnVisitList.forEach(i -> returnVisitMap.put(i.getSourceDataId(), i));
            }
        }

        DateTime now = DateUtil.date();
        // 5.数据整理
        List<FollowUpDetailQueryBO> boList = new ArrayList<>();
        followUpList.forEach(i -> {
            i.setStatus(YN.YES.getCode());
            if (DateUtil.offsetDay(i.getPlanFollowTime(), maxDays).after(now)) {
                i.setStatus(YN.NO.getCode());
            }

            FollowUpDetailQueryBO returnVisit = returnVisitMap.get(i.getId());
            if (null != returnVisit) {
                i.setFollowVisitDeptCode(returnVisit.getFollowVisitDeptCode());
                i.setFollowVisitStatus(returnVisit.getFollowVisitStatus());
            }

            if (YN.YES.getCode().equals(i.getFollowVisitStatus())) {
                i.setStatus(YN.YES.getCode());
            }
            
            boList.add(i);
        });
        
        // 数据是否存在
        Map<String, FollowUpDetailInfo> existMap = new HashMap<>();
        if (CollUtil.isNotEmpty(boList)) {
            List<String> existParams = new ArrayList<>();
            boList.forEach(i -> {
                existParams.add(i.getSourceDataId());
            });
            if (CollUtil.isNotEmpty(existParams)) {
                List<FollowUpDetailInfo> existList = followUpDetailInfoService.list(new QueryWrapper<FollowUpDetailInfo>()
                    .in("source_data_id", existParams)
                );
                if (CollUtil.isNotEmpty(existList)) {
                    existList.forEach(i -> {
                        existMap.put(i.getSourceDataId(), i);
                    });
                }
            }
        }

        // 保存
        List<FollowUpDetailInfo> saveList = new ArrayList<>();
        List<FollowUpDetailInfo> updateList = new ArrayList<>();
        if (CollUtil.isNotEmpty(boList)) {
            boList.forEach(i -> {
                FollowUpDetailInfo info = existMap.get(i.getSourceDataId());
                if (null == info) {
                    info = new FollowUpDetailInfo();
                    info.setId(IdUtil.getSnowflake(1,1).nextIdStr());
                    saveList.add(info);
                } else {
                    updateList.add(info);
                }
                info.setDeptCode(i.getDeptCode());
                info.setPatientId(i.getPatientId());
                info.setSourceType(i.getSourceType());
                info.setSourceDataId(i.getSourceDataId());
                info.setSourceDataTime(i.getSourceDataTime());
                info.setPlanFollowTime(i.getPlanFollowTime());
                info.setFollowExeStatus(i.getFollowExeStatus());
                info.setFollowExeTime(i.getFollowExeTime());
                info.setFollowVisitStatus(i.getFollowVisitStatus());
                info.setFollowVisitDeptCode(i.getFollowVisitDeptCode());
                info.setStatus(i.getStatus());
                info.setLatestTime(now);
            });
        }

        if (CollUtil.isNotEmpty(saveList)) {
            followUpDetailInfoService.saveBatch(saveList);
        }
        if (CollUtil.isNotEmpty(updateList)) {
            followUpDetailInfoService.updateBatchById(updateList);
        }
    }

    @Override
    public void followUpDetailUpdateStatisticsByDate(String startDate, String endDate) {
        DateTime now = DateUtil.parseDate(DateUtil.formatDate(DateUtil.date()));
        
        if (StrUtil.isBlank(startDate)) {
            startDate = DateUtil.offsetDay(now, -(maxDays + 1)).toDateStr();
        }
        DateTime start = DateUtil.parseDate(startDate);
        DateTime finishDate = DateUtil.offsetDay(now, maxDays);
        if (StrUtil.isNotBlank(endDate)) {
            finishDate = DateUtil.parseDate(endDate);
        }
        while (true) {
            if (start.after(finishDate)) {
                return;
            }
            DateTime end = DateUtil.offsetDay(start, 1);
            
            followUpDetailUpdateStatisticsByDateHander(start.toDateStr(), end.toDateStr());
            start = end;
        }
    }
    
    public void followUpDetailUpdateStatisticsByDateHander(String startDate, String endDate) {
        // 按指定配置的时间查询
        List<FollowUpDetailInfo> list = followUpDetailInfoService.list(new QueryWrapper<FollowUpDetailInfo>()
            .eq("status", YN.NO.getCode()) 
            .ge("plan_follow_time", startDate) // 计划时间
            .lt("plan_follow_time", endDate)
        );

        if (CollUtil.isEmpty(list)) {
            return;
        }

        // 1.随访数据
        Set<String> sourceIds = new HashSet<>();
        list.forEach(i -> {
            if (StrUtil.isNotBlank(i.getSourceDataId())) {
                sourceIds.add(i.getSourceDataId());
            }
        });

        // DateTime maxDayDate = DateUtil.offsetDay(DateUtil.parseDate(startDate), maxDays);
        
        Map<String, FollowUpDetailQueryBO> followUpMap = new HashMap<>();
        if (CollUtil.isNotEmpty(sourceIds)) {
            List<FollowUpDetailQueryBO> followUpList = statisticsDataQueryByBizMapper.queryFollowUpDateInfo(new QueryWrapper<Void>()
                .in("id", sourceIds)
                // .ge("source_date", startDate)
                // .lt("source_date", maxDayDate)
            );
            if (CollUtil.isNotEmpty(followUpList)) {
                followUpList.forEach(i -> {
                    followUpMap.put(i.getId(), i);
                });
            }
        }
        
        // 复诊
        Map<String, FollowUpDetailQueryBO> returnVisitMap = new HashMap<>();
        if (CollUtil.isNotEmpty(sourceIds)) {
            List<FollowUpDetailQueryBO> returnVisitList = statisticsDataQueryByBizMapper.queryReturnVisitInfo(new QueryWrapper<Void>()
                .eq("source_type", "follow_up")
                .in("source_id", sourceIds)
            );
            if (CollUtil.isNotEmpty(returnVisitList)) {
                returnVisitList.forEach(i -> returnVisitMap.put(i.getSourceDataId(), i));
            }
        }

        Date now = DateUtil.date();
        // 更新
        list.forEach(i -> {
            i.setStatus(YN.YES.getCode());
            if (DateUtil.offsetDay(i.getPlanFollowTime(), maxDays).after(now)) {
                i.setStatus(YN.NO.getCode());
            }
            
            FollowUpDetailQueryBO followup = followUpMap.get(i.getSourceDataId());
            if (null != followup) {
                i.setFollowExeStatus(followup.getFollowExeStatus());
                i.setFollowExeTime(followup.getFollowExeTime());

                FollowUpDetailQueryBO returnVisit = returnVisitMap.get(followup.getId());
                if (null != returnVisit) {
                    i.setFollowVisitStatus(returnVisit.getFollowVisitStatus());
                    i.setFollowVisitTime(returnVisit.getFollowVisitTime());
                }
            }
            
            if (YN.YES.getCode().equals(i.getFollowVisitStatus())) {
                i.setStatus(YN.YES.getCode());
            }

            i.setLatestTime(now);
        });

        followUpDetailInfoService.updateBatchById(list);
    }

    @Override
    public void followUpDayStatisticsByDate(String startDate, String endDate) {
        DateTime now = DateUtil.date();
        
        if (StrUtil.isBlank(startDate)) {
            startDate = DateUtil.offsetDay(now, -maxDays).toDateStr();
        }
        DateTime start = DateUtil.parseDate(startDate);
        DateTime finishDate = DateUtil.offsetDay(now, 1);
        if (StrUtil.isNotBlank(endDate)) {
            finishDate = DateUtil.parseDate(endDate);
        }

        // 循环每天处理
        while (true) {
            if (start.after(finishDate)) {
                return;
            }
            DateTime end = DateUtil.offsetDay(start, 1);

            followUpDayStatisticsByDateHander(start.toDateStr(), end.toDateStr());
            start = end;
        }
    }

    public void followUpDayStatisticsByDateHander(String startDate, String endDate) {
        if (StrUtil.isBlank(startDate)) {
            return;
        }
        
        List<FollowUpDayQueryBO> list = statisticsDataQueryByBizMapper.queryFollowUpDayList(new QueryWrapper<Void>()
            .ge("source_data_time", startDate)
            .lt(StrUtil.isNotBlank(endDate), "source_data_time", endDate)
            .groupBy("deptCode", "dataTime")
        );
        if(CollUtil.isEmpty(list)){
            return;
        }

        Set<String> dateList = new HashSet<>();
        list.forEach(item -> {
            dateList.add(item.getDataTime());
        });

        // 查询已统计数据
        Map<String, FollowUpDayInfo> existMap = new HashMap<>();
        List<FollowUpDayInfo> existList = followUpDayInfoService.list(new QueryWrapper<FollowUpDayInfo>()
            .in("data_time", dateList)
        );
        if (CollUtil.isNotEmpty(existList)) {
            existList.forEach(i -> existMap.put(i.getDataTime() + "_" + i.getDeptCode(), i));
        }

        Date latestTime = new Date();

        List<FollowUpDayInfo> saveList = new ArrayList<>();
        List<FollowUpDayInfo> updateList = new ArrayList<>();

        list.forEach(item->{
            FollowUpDayInfo entity = existMap.get(item.getDataTime() + "_" + item.getDeptCode());
            if (null == entity) {
                entity = new FollowUpDayInfo();
                entity.setId(item.getDeptCode() + "_" + item.getDataTime());
                saveList.add(entity);
            } else {
                updateList.add(entity);
            }
            entity.setDataTime(item.getDataTime());
            entity.setDeptCode(item.getDeptCode());
            entity.setTotalNum(item.getTotalNum());
            entity.setExeYesNum(item.getExeYesNum());
            entity.setExeNoNum(item.getExeNoNum());
            entity.setVisitNum(item.getVisitNum());
            entity.setLatestTime(latestTime);

        });

        if (CollUtil.isNotEmpty(saveList)) {
            followUpDayInfoService.saveBatch(saveList);
        }
        if (CollUtil.isNotEmpty(updateList)) {
            followUpDayInfoService.updateBatchById(updateList);
        }    
    }
    
}
