package com.xxh.cloud.data.statistics.framework.modules.task.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.PatientFollowupDayInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.PatientFollowupDetailInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.service.PatientFollowupDayInfoService;
import com.xxh.cloud.data.statistics.framework.modules.base.service.PatientFollowupDetailInfoService;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.PatientFollowupDayQueryBO;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.PatientFollowupQueryBO;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.PatientVisitRecordBO;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.dto.PatientVisitRecordQueryDTO;
import com.xxh.cloud.data.statistics.framework.modules.task.mapper.StatisticsDataQueryByBizMapper;
import com.xxh.cloud.data.statistics.framework.modules.task.service.PatientFollowupDayTaskService;
import com.xxh.cloud.data.statistics.framework.modules.task.service.StatisticsDwdDataQueryService;
import com.xxh.cloud.data.statistics.framework.modules.task.utils.DataStatisticsTimeUtils;
import com.xxh.cloud.framework.common.constant.YN;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

@Service
public class PatientFollowupDayTaskServiceImpl implements PatientFollowupDayTaskService {

    @Autowired
    private PatientFollowupDayInfoService patientFollowupDayInfoService;
    @Autowired
    private PatientFollowupDetailInfoService patientFollowupDetailInfoService;
    @Autowired
    private StatisticsDwdDataQueryService statisticsDwdDataQueryService;
    @Resource
    private StatisticsDataQueryByBizMapper statisticsDataQueryByBizMapper;

    /**
     *
     * */
    private void sync(String date){
        List<PatientFollowupQueryBO> list= statisticsDataQueryByBizMapper.queryPatientFollowup(
                new QueryWrapper<PatientFollowupQueryBO>()
                        .eq("DATE_FORMAT(visit_start_time, '%Y-%m-%d')",date)
//                        .eq("status", "follow_up")
        );
        if(CollUtil.isEmpty(list)){
            return;
        }

        List<PatientFollowupDetailInfo> saveList=new ArrayList<>();

        for (PatientFollowupQueryBO dataItem:list){
            PatientFollowupDetailInfo saveEntity=new PatientFollowupDetailInfo();
            saveEntity.setDataTime(date);
            saveEntity.setId(IdUtil.getSnowflake(2,1).nextIdStr());
            saveEntity.setPatientId(dataItem.getPatientId());
            saveEntity.setMainPatientId(dataItem.getMainPatientId());
            saveEntity.setDiscoveryTime(dataItem.getDiscoveryTime());
            saveEntity.setRelSourceType("PATIENT_VISIT"); // 患者复诊表
            saveEntity.setRelSourceId(dataItem.getId());
            saveEntity.setVisitStartTime(dataItem.getVisitStartTime());
            saveEntity.setVisitEndTime(dataItem.getVisitEndTime());
            saveEntity.setAppointDeptCode(dataItem.getDeptIds());
            saveEntity.setIsFollowup(YN.NO.getCode());
            saveEntity.setLatestTime(new Date());
            saveList.add(saveEntity);
        }

        patientFollowupDetailInfoService.remove(
                new QueryWrapper<PatientFollowupDetailInfo>()
                        .eq("data_time",date)
        );
        patientFollowupDetailInfoService.saveBatch(saveList);

    }

    /**
     *
     * */
    private Set<String> calculate(String startDate, String endDate) {
        Set<String> syncDate = new HashSet<>();
        // 查询时间段内复诊人数
        List<PatientFollowupDetailInfo> list = patientFollowupDetailInfoService.list(
                new QueryWrapper<PatientFollowupDetailInfo>()
                        .ge("visit_end_time", startDate)        // 复诊未过期的
                        .eq("is_followup",YN.NO.getCode())
        );

        if(CollUtil.isEmpty(list)){
            return syncDate;
        }

        Set<String> patientIds=new HashSet<>();
        list.forEach(item->{
            patientIds.add(item.getPatientId());
        });

        // 查询诊断记录
        PatientVisitRecordQueryDTO dto = new PatientVisitRecordQueryDTO();
        dto.setPatientIds(patientIds);
        dto.setStartDate(startDate);
        dto.setEndDate(endDate);
        List<PatientVisitRecordBO> patientVisitRecords = statisticsDwdDataQueryService.getPatientVisitRecordList(dto);
        if(CollUtil.isEmpty(patientVisitRecords)){
            return syncDate;
        }
        patientVisitRecords.forEach(item->{
            // 查询患者的复诊记录
            List<PatientFollowupDetailInfo> detailList = patientFollowupDetailInfoService.list(new QueryWrapper<PatientFollowupDetailInfo>()
                    .eq("patient_id", item.getPatientId())
                    .eq("is_followup",YN.NO.getCode())
                    .and(i -> i.like("appoint_dept_code", item.getDepartmentCode()).or().isNull("appoint_dept_code"))
            );
            if (CollUtil.isNotEmpty(detailList)) {
                for (PatientFollowupDetailInfo detailInfo : detailList) {
                    syncDate.add(detailInfo.getDataTime());
                    detailInfo.setIsFollowup(YN.YES.getCode());
                    detailInfo.setFollowupDeptCode(item.getDepartmentCode());
                    detailInfo.setFollowupTime(item.getVisitDate());
                    detailInfo.setLatestTime(new Date());

                    // 更新h_patient_visit_info表，复诊确认
                    statisticsDataQueryByBizMapper.updatePatientReturnVisit(new UpdateWrapper<Void>()
                            .set("dept_id", detailInfo.getFollowupDeptCode())
                            .set("real_time", item.getVisitDate())
                            .set("status", "return_visit")   // 已复诊
                            .eq("id", detailInfo.getRelSourceId())
                    );
                }
                patientFollowupDetailInfoService.updateBatchById(detailList);
            }
        });
        return syncDate;
    }

    /**
     *
     * */
    private void statistics(String date){
        PatientFollowupDayQueryBO bo = statisticsDataQueryByBizMapper.queryPatientFollowupDay(
                new QueryWrapper<PatientFollowupDayQueryBO>()
                        .eq("data_time",date)
        );

        if(null == bo){
            return;
        }

        PatientFollowupDayInfo entity = new PatientFollowupDayInfo();
        entity.setId(IdUtil.getSnowflake(2,1).nextIdStr());
        entity.setDataTime(date);
        entity.setFollowupNum(bo.getFollowupNum());
        entity.setFollowupLatestTime(new Date());
        entity.setFollowupYesNum(bo.getFollowupYesNum());
        entity.setFollowupYesLatestTime(new Date());
        patientFollowupDayInfoService.remove(new QueryWrapper<PatientFollowupDayInfo>()
                        .eq("data_time",date)
        );

        patientFollowupDayInfoService.save(entity);
    }

    @Override
    public void syncFollowupPatient(String date) {
        this.sync(date);
        this.statistics(date);
    }

    @Override
    public void calculateFollowup(String date) {
        //
        Date date1= DataStatisticsTimeUtils.str2Date(date);
        String begin= DataStatisticsTimeUtils.beginOfDay(date1);
        String end= DataStatisticsTimeUtils.endOfDay(date1);
        //计算诊断-统计明细表
        Set<String> syncDate = this.calculate(begin,end);
        //统计日统计表
        syncDate.forEach(this::statistics);
    }
}
