package com.xxh.cloud.data.statistics.framework.modules.task.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.PatientInpatientLossDayInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.PatientInpatientLossDetailInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.service.PatientInpatientLossDayInfoService;
import com.xxh.cloud.data.statistics.framework.modules.base.service.PatientInpatientLossDetailInfoService;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.PatientFollowupDayQueryBO;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.PatientInpatientApplyQueryBO;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.PatientInpatientRecordQueryBO;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.PatientPatientInpatientLossQueryBO;
import com.xxh.cloud.data.statistics.framework.modules.task.mapper.StatisticsDataQueryByBizMapper;
import com.xxh.cloud.data.statistics.framework.modules.task.service.PatientInpatientLossDayTaskService;
import com.xxh.cloud.data.statistics.framework.modules.task.service.StatisticsDwdDataQueryService;
import com.xxh.cloud.data.statistics.framework.modules.task.utils.DataStatisticsEntityIdUtils;
import com.xxh.cloud.data.statistics.framework.modules.task.utils.DataStatisticsTimeUtils;
import com.xxh.cloud.framework.common.constant.YN;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class PatientInpatientLossDayTaskServiceImpl implements PatientInpatientLossDayTaskService {

    @Autowired
    private PatientInpatientLossDayInfoService patientInpatientLossDayInfoService;
    @Autowired
    private PatientInpatientLossDetailInfoService patientInpatientLossDetailInfoService;
    @Autowired
    private StatisticsDwdDataQueryService statisticsDwdDataQueryService;
    @Autowired
    private StatisticsDataQueryByBizMapper statisticsDataQueryByBizMapper;


    private PatientInpatientLossDetailInfo getEntity(PatientInpatientApplyQueryBO item){
        PatientInpatientLossDetailInfo entity=new PatientInpatientLossDetailInfo();
        entity.setId(DataStatisticsEntityIdUtils.getInpatientApplyId(item.getPatientId(), String.valueOf(item.getApplyDate().getTime())));
        entity.setPatientId(item.getPatientId());
        entity.setOpenInpatientTime(item.getApplyDate());
        entity.setOpenInpatientDeptCode(item.getApplyDepartmentCode());
        entity.setIsInpatient(YN.NO.getCode());
        entity.setLatestTime(new Date());
        return entity;
    }

    /** 同步住院申请信息明细 */
    private void sync(String date){
        List<PatientInpatientApplyQueryBO> list= statisticsDwdDataQueryService.getPatientInpatientApplyList(date);
        if(CollUtil.isEmpty(list)){
            return;
        }
        List<PatientInpatientLossDetailInfo> entityList = new ArrayList<>();
        list.forEach(item->{
            entityList.add(this.getEntity(item));
        });

        if(CollUtil.isNotEmpty(entityList)){
            patientInpatientLossDetailInfoService.saveOrUpdateBatch(entityList);
        }
    }

    /** 更新住院申请信息明细 */
    private Set<String> calculate(String startDate, String endDate) {
        Set<String> statisticsDates = new HashSet<>();
        List<PatientInpatientRecordQueryBO> list= statisticsDwdDataQueryService.getPatientInpatientRecordList(startDate,endDate);
        if(CollUtil.isEmpty(list)){
            return statisticsDates;
        }
        list.forEach(item->{
            Date dayBf15 = DateUtil.offsetDay(item.getInpatientDate(), -15);
            List<PatientInpatientLossDetailInfo> entityList = patientInpatientLossDetailInfoService.list(new QueryWrapper<PatientInpatientLossDetailInfo>()
                    .eq("patient_id",item.getPatientId())
                    .ge("open_inpatient_time", dayBf15));
            if (CollUtil.isEmpty(entityList)) {
                return;
            }
            for (PatientInpatientLossDetailInfo info : entityList) {
                statisticsDates.add(DataStatisticsTimeUtils.getYYYYMMDD(info.getOpenInpatientTime()));
                info.setIsInpatient(YN.YES.getCode());
                info.setInpatientDeptCode(item.getDepartmentCode());
                info.setInpatientTime(item.getInpatientDate());
                info.setLatestTime(new Date());
            }
            patientInpatientLossDetailInfoService.updateBatchById(entityList);
        });

        return statisticsDates;
    }

    /**
     * 计算住院统计
     * */
    private void statistics(String date) {
        PatientPatientInpatientLossQueryBO bo = statisticsDataQueryByBizMapper.queryPatientInpatientLossDay(
                new QueryWrapper<PatientPatientInpatientLossQueryBO>()
                        .eq("DATE_FORMAT(open_inpatient_time, '%Y-%m-%d')",date)
        );

        if(null == bo){
            return;
        }

        PatientInpatientLossDayInfo entity = new PatientInpatientLossDayInfo();
        entity.setId(IdUtil.getSnowflake(2,1).nextIdStr());
        entity.setDataTime(date);
        entity.setNum(bo.getNum());
        entity.setLatestTime(new Date());
        entity.setInpatientNum(bo.getInpatientNum());
        entity.setLatestInpatientTime(new Date());
        patientInpatientLossDayInfoService.remove(new QueryWrapper<PatientInpatientLossDayInfo>()
                .eq("data_time",date)
        );
        patientInpatientLossDayInfoService.save(entity);
    }



    @Override
    public void syncInpatientApply(String date) {
        this.sync(date);
        this.statistics(date);
    }

    @Override
    public void calculateInpatientApply(String date) {
        //
        Date date1= DataStatisticsTimeUtils.str2Date(date);
        String begin= DataStatisticsTimeUtils.beginOfDay(date1);
        String end= DataStatisticsTimeUtils.endOfDay(date1);
        //计算诊断-统计明细表
        Set<String> statisticsDate = this.calculate(begin,end);
        //统计日统计表
        statisticsDate.forEach(this::statistics);
    }
}
