package com.xxh.cloud.data.statistics.framework.modules.task.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.PotentialPatientNotDiagnosisDayInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.PotentialPatientNotDiagnosisDetailInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.service.PotentialPatientNotDiagnosisDayInfoService;
import com.xxh.cloud.data.statistics.framework.modules.base.service.PotentialPatientNotDiagnosisDetailInfoService;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.DiseaseIcdRelBO;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.PatientDiagnosisByPatientQueryBO;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.PatientLabelDayQueryBO;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.PotentialPatientNotDiagnosisDayQueryBO;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.dto.PatientDiagnosisByPatientQueryDTO;
import com.xxh.cloud.data.statistics.framework.modules.task.mapper.StatisticsDataQueryByBizMapper;
import com.xxh.cloud.data.statistics.framework.modules.task.service.PotentialPatientNotDiagnosisDayTaskService;
import com.xxh.cloud.data.statistics.framework.modules.task.service.StatisticsDwdDataQueryService;
import com.xxh.cloud.data.statistics.framework.modules.task.utils.DataStatisticsTimeUtils;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class PotentialPatientNotDiagnosisDayTaskServiceImpl implements PotentialPatientNotDiagnosisDayTaskService {


    @Autowired
    private PotentialPatientNotDiagnosisDayInfoService potentialPatientNotDiagnosisDayInfoService;

    @Autowired
    private PotentialPatientNotDiagnosisDetailInfoService potentialPatientNotDiagnosisDetailInfoService;

    @Resource
    private StatisticsDataQueryByBizMapper statisticsDataQueryByBizMapper;

    @Autowired
    private StatisticsDwdDataQueryService statisticsDwdDataQueryService;

    private void sync(String date){

        // 标签分类
        // Tags_Type_Data("tags_type_data","数据标签"),
        // Tags_Type_Action("tags_type_action","行为标签"),
        //Tags_Type_Statistics("tags_type_statistics","统计标签"),

        //诊前
        //LSD_Pre_Not_Visit("pre_notVisit","发现潜在患者/未就诊"),
        //LSD_Pre_Visited("pre_visited","发现潜在患者/已就诊"),
        //LSD_Pre_Confirmed("pre_confirmed","发现潜在患者/已确诊"),
        //诊中
        //LSD_During_Register("during_register","挂号流失患者"),
        //LSD_During_InHospital("during_inHospital","住院流失患者"),
        //LSD_During_Operation("during_operation","手术流失患者"),
        //诊后
        //LSD_After_Lost ("after_lost","失访患者"),

        List<PatientLabelDayQueryBO> list= statisticsDataQueryByBizMapper.queryPatientLabelDay(
                new QueryWrapper<PatientLabelDayQueryBO>()
                        .eq("a.op_type","1")
                        .eq("DATE_FORMAT(a.data_dt, '%Y-%m-%d')",date)
                        //.eq("b.biz_type","")
                        .in("b.stat_dimension_type",
                                Arrays.asList("pre_notVisit", "pre_visited", "pre_confirmed"))
                        .groupBy("a.patient_id","a.main_patient_id","c.disease_id")

        );
        if(CollUtil.isEmpty(list)){
            return;
        }

        List<PotentialPatientNotDiagnosisDetailInfo> saveList=new ArrayList<>();

        for (PatientLabelDayQueryBO dataItem:list){
            PotentialPatientNotDiagnosisDetailInfo saveEntity=new PotentialPatientNotDiagnosisDetailInfo();
            saveEntity.setId(IdUtil.getSnowflake(2,1).nextIdStr());
            saveEntity.setDataTime(date);
            saveEntity.setPatientId(dataItem.getPatientId());
            saveEntity.setMainPatientId(dataItem.getMainPatientId());
            saveEntity.setDiseaseId(dataItem.getDiseaseId());
            saveEntity.setDiscoveryTime(dataItem.getDataTime());
            saveEntity.setIsDiagnosis(YN.NO.getCode());
            saveEntity.setLatestTime(new Date());
            saveList.add(saveEntity);
        }

        potentialPatientNotDiagnosisDetailInfoService.remove(
                new QueryWrapper<PotentialPatientNotDiagnosisDetailInfo>()
                        .eq("data_time",date)
        );


        potentialPatientNotDiagnosisDetailInfoService.saveBatch(saveList);

    }


    private void statistics(String date){
        List<PotentialPatientNotDiagnosisDayQueryBO> list= statisticsDataQueryByBizMapper.queryPotentialPatientNotDiagnosisDay(
                new QueryWrapper<PotentialPatientNotDiagnosisDayQueryBO>()
                        .eq("data_time",date)
                        .groupBy("disease_id")
        );

        if(CollUtil.isEmpty(list)){
            return;
        }

        List<PotentialPatientNotDiagnosisDayInfo> saveList=new ArrayList<>();

        for(PotentialPatientNotDiagnosisDayQueryBO item:list){
            PotentialPatientNotDiagnosisDayInfo saveItem=new PotentialPatientNotDiagnosisDayInfo();
            saveItem.setId(IdUtil.getSnowflake(2,1).nextIdStr());
            saveItem.setDataTime(date);
            saveItem.setDiseaseId(item.getDiseaseId());
            saveItem.setNum(item.getTotalNum());
            saveItem.setDiagnosisNum(item.getDiagnosisNum());
            saveItem.setLatestTime(new Date());
            saveList.add(saveItem);
        }

        potentialPatientNotDiagnosisDayInfoService.remove(
                new QueryWrapper<PotentialPatientNotDiagnosisDayInfo>()
                        .eq("data_time",date)
        );

        potentialPatientNotDiagnosisDayInfoService.saveBatch(saveList);

    }


    private void calculate(String startDate ,String endDate){
        List<PotentialPatientNotDiagnosisDetailInfo> list= potentialPatientNotDiagnosisDetailInfoService.list(
                new QueryWrapper<PotentialPatientNotDiagnosisDetailInfo>()
                        .ge("discovery_time", startDate)
                        .le("discovery_time", endDate)
                        .eq("is_diagnosis",YN.NO.getCode())
        );

        if(CollUtil.isEmpty(list)){
            return;
        }

        Set<String> patientIds=new HashSet<>();
        Set<String> diseaseIds=new HashSet<>();
        list.forEach(item->{
            patientIds.add(item.getPatientId());
            diseaseIds.add(item.getDiseaseId());
        });

        // 查询诊断记录
        PatientDiagnosisByPatientQueryDTO dto = new PatientDiagnosisByPatientQueryDTO();
        dto.setPatientIds(patientIds);
        dto.setStartDate(startDate);
        dto.setEndDate(endDate);
        List<PatientDiagnosisByPatientQueryBO> diagnosisList= statisticsDwdDataQueryService.getPatientDiagnosisList(dto);
        if(CollUtil.isEmpty(diagnosisList)){
            return;
        }

        Map<String,PatientDiagnosisByPatientQueryBO> diagnosisMap=new HashMap<>();
        diagnosisList.forEach(item->{
            diagnosisMap.put(item.getPatientId(),item);
        });

        // 病种关联ICD
        List<DiseaseIcdRelBO> diseaseIcdRel = statisticsDataQueryByBizMapper.queryDiseaseIcdRelConcat(new QueryWrapper<DiseaseIcdRelBO>()
                .eq("rows_status", YN.YES.getCode())
                .in("disease_id", diseaseIds)
                .groupBy("disease_id")
        );

        Map<String,List<String>> diseaseIcdMap = new HashMap<>();
        diseaseIcdRel.forEach(item->{
            diseaseIcdMap.put(item.getDiseaseId(), StrJoinSplitUtils.split2List(item.getIcd10Codes()));
        });


        List<String> yesDiagnosisIds=new ArrayList<>();

        for(PotentialPatientNotDiagnosisDetailInfo diagnosisDetailItem:list){
            PatientDiagnosisByPatientQueryBO patientDiagnosisByPatientQueryBO =  diagnosisMap.get(diagnosisDetailItem.getPatientId());
            if(patientDiagnosisByPatientQueryBO!=null){
                List<String> diagnosisCodes = StrJoinSplitUtils.split2List(patientDiagnosisByPatientQueryBO.getDiagnosisCodes());
                List<String> diseaseIcdList = diseaseIcdMap.get(diagnosisDetailItem.getDiseaseId());
                // 病种的交集
                List<String> intersection = diagnosisCodes.stream().filter(diseaseIcdList::contains).collect(Collectors.toList());
                if (CollUtil.isNotEmpty(intersection)) {
                    yesDiagnosisIds.add(diagnosisDetailItem.getId());
                }
            }
        }

        if(yesDiagnosisIds.size()>0){
            potentialPatientNotDiagnosisDetailInfoService.update(
                    new UpdateWrapper<PotentialPatientNotDiagnosisDetailInfo>()
                            .set("is_diagnosis",YN.YES.getCode())
                            .set("latest_time",new Date())
                            .in("id",yesDiagnosisIds)
            );
        }

    }


    @Override
    public void syncPotentialPatient(String date) {
        //同步统计明细表
        this.sync(date);
        //统计日统计表
        this.statistics(date);
    }

    @Override
    public void calculateDiagnosis(String date) {

        Date date1= DataStatisticsTimeUtils.str2Date(date);
        String begin= DataStatisticsTimeUtils.beginOfDay(date1);
        String end= DataStatisticsTimeUtils.endOfDay(date1);
        //计算诊断-统计明细表
        this.calculate(begin,end);
        //统计日统计表
        this.statistics(date);

    }

}
