package com.xxh.cloud.data.statistics.framework.modules.task.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.PushDayInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.PushDetailInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.service.PushDayInfoService;
import com.xxh.cloud.data.statistics.framework.modules.base.service.PushDetailInfoService;
import com.xxh.cloud.data.statistics.framework.modules.constant.PushConstant;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.PushDayQueryBO;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.PushDetailQueryBO;
import com.xxh.cloud.data.statistics.framework.modules.task.mapper.StatisticsDataQueryByBizMapper;
import com.xxh.cloud.data.statistics.framework.modules.task.service.PushServeTaskService;
import com.xxh.cloud.framework.common.constant.YN;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 服务详情日统计
 * */
@Service
public class PushServeTaskServiceImpl implements PushServeTaskService {

    @Autowired
    private StatisticsDataQueryByBizMapper statisticsDataQueryByBizMapper;
    @Autowired
    private PushDetailInfoService pushDetailInfoService;
    @Autowired
    private PushDayInfoService pushDayInfoService;

    @Value("${push.maxDays:7}")
    private Integer maxDays; // 以服务推送时间为基准

    @Override
    public void pushDetailStatisticsByDate(String startDate, String endDate) {
        DateTime now = DateUtil.date();
        DateTime start = DateUtil.parseDate(startDate);
        DateTime finishDate = DateUtil.offsetDay(now, 1);
        if (StrUtil.isNotBlank(endDate)) {
            finishDate = DateUtil.parseDate(endDate);
        }

        // 循环每天处理
        while (true) {
            DateTime end = DateUtil.offsetDay(start, 1);
            if (end.after(finishDate)) {
                return;
            }
            String status = YN.YES.getCode();
            if (DateUtil.offsetDay(start, maxDays).after(now)) { // 若推送服务最大数据变化日期大于当前日期则说明该数据需要后续更新
                status = YN.NO.getCode();
            }
            pushDetailStatisticsByDateHander(start.toDateStr(), end.toDateStr(), status);
            start = end;
        }
    }

    public void pushDetailStatisticsByDateHander(String startDate, String endDate, String status) {
        if (StrUtil.isBlank(startDate)) {
            return;
        }
        
        // 1.查询患者服务推送日期
        List<PushDetailQueryBO> patientPushServeDateList = statisticsDataQueryByBizMapper.queryPushServeDateInfo(new QueryWrapper<Void>()
            .ge("send_date", startDate)
            .lt("send_date", endDate)
        );

        if (CollUtil.isEmpty(patientPushServeDateList)) {
            return;
        }

        List<String> patientPushServeIds = new ArrayList<>();
        patientPushServeDateList.forEach(item -> {
            patientPushServeIds.add(item.getServId());
        });

        // 2.根据服务查询推送服务
        Map<String, PushDetailQueryBO> patientPushServeMap = new HashMap<>();
        List<PushDetailQueryBO> patientPushServeList = statisticsDataQueryByBizMapper.queryPushServeInfo(new QueryWrapper<Void>()
            .eq("op_type", "1")
            .in("patient_push_serve_id", patientPushServeIds)
        );
        if (CollUtil.isNotEmpty(patientPushServeList)) {
            patientPushServeList.forEach(i -> {
                patientPushServeMap.put(i.getServId(), i);
            });
        }

        // 3.根据服务查询提醒
        List<String> remindIds = new ArrayList<>();
        Map<String, List<PushDetailQueryBO>> remindMap = new HashMap<>();
        List<PushDetailQueryBO> remindList = statisticsDataQueryByBizMapper.queryPushRemindInfo(new QueryWrapper<Void>()
            .eq("rows_status", YN.YES.getCode())
            .in("biz_id", patientPushServeIds)
        );
        if (CollUtil.isNotEmpty(remindList)) {
            remindList.forEach(i -> {
                remindIds.add(i.getRemindId());

                List<PushDetailQueryBO> l = remindMap.get(i.getServId() + "_" + i.getPushTime());
                if (null == l) {
                    l = new ArrayList<>();
                    remindMap.put(i.getServId() + "_" + i.getPushTime(), l);
                }
                
                l.add(i);
            });
        }

        // 4.根据提醒查询患者行为
        Map<String, PushDetailQueryBO> patientPushServeBehaviorMap = new HashMap<>();
        if (CollUtil.isNotEmpty(remindIds)) {
            List<PushDetailQueryBO> patientPushServeBehaviorList = statisticsDataQueryByBizMapper.queryPushServeBehaviorInfo(new QueryWrapper<Void>()
                .eq("source_one_type", "remind")
                .in("source_one_id", remindIds)
                .groupBy("source_one_id")
            );
            if (CollUtil.isNotEmpty(patientPushServeBehaviorList)) {
                patientPushServeBehaviorList.forEach(i -> {
                    patientPushServeBehaviorMap.put(i.getRemindId(), i);
                });
            }
        }

        // 4.整理患者服务数据
        List<PushDetailQueryBO> boList = new ArrayList<>();
        patientPushServeDateList.forEach(i -> {
            i.setStatus(status);
            // 服务
            PushDetailQueryBO patientPushServe = patientPushServeMap.get(i.getServId());
            if (null == patientPushServe) {
                return;
            }
            i.setServConfigId(patientPushServe.getServConfigId());
            i.setServTime(patientPushServe.getServTime());
            // 提醒
            List<PushDetailQueryBO> rl = remindMap.get(i.getServId() + "_" + i.getPushTime());
            if (CollUtil.isNotEmpty(rl)) {
                rl.forEach(j -> {
                    j.setServConfigId(i.getServConfigId());
                    j.setReadStatus(YN.NO.getCode());
                    j.setActionStatus(YN.NO.getCode());
                    // j.setPushStatus(j.getPushStatus());
                    j.setStatus(i.getStatus());
                    
                    boList.add(j);

                    // 行为
                    if (PushConstant.PushStatus.Success.getCode().equals(j.getPushStatus())) {
                        PushDetailQueryBO behavior = patientPushServeBehaviorMap.get(j.getRemindId());
                        if (null != behavior) {
                            if (YN.YES.getCode().equals(behavior.getReadStatus())) {
                                j.setReadStatus(YN.YES.getCode());
                                j.setReadTime(behavior.getReadTime());
                            }

                            if (YN.YES.getCode().equals(behavior.getActionStatus())) {
                                j.setActionStatus(YN.YES.getCode());
                                j.setActionTime(behavior.getActionTime());
                            }

                            if (YN.YES.getCode().equals(j.getReadStatus()) && YN.YES.getCode().equals(j.getActionStatus())) {
                                j.setStatus(YN.YES.getCode());
                            }
                        }
                    }
                });
            } else {
                i.setPushStatus(PushConstant.PushStatus.NO_FAMILY.getCode());
                i.setReadStatus(YN.NO.getCode());
                i.setActionStatus(YN.NO.getCode());
                boList.add(i);
            }
        });

        // 数据是否存在
        Map<String, PushDetailInfo> existMap = new HashMap<>();
        Map<String, PushDetailInfo> existPushMap = new HashMap<>();
        if (CollUtil.isNotEmpty(boList)) {
            List<String> existParams = new ArrayList<>();
            List<String> existPushParams = new ArrayList<>();
            boList.forEach(i -> {
                if (StrUtil.isNotBlank(i.getRemindId())) {
                    existParams.add(i.getRemindId());
                } else {
                    existPushParams.add(i.getServId()); // 查询没有家属的服务，若处理时该服务有家属则需要删除该数据
                }
            });
            if (CollUtil.isNotEmpty(existParams)) {
                List<PushDetailInfo> existList = pushDetailInfoService.list(new QueryWrapper<PushDetailInfo>()
                    .in("remind_id", existParams)
                );
                if (CollUtil.isNotEmpty(existList)) {
                    existList.forEach(i -> {
                        existMap.put(i.getRemindId(), i);
                    });
                }
            }
            if (CollUtil.isNotEmpty(existPushParams)) {
                List<PushDetailInfo> existList = pushDetailInfoService.list(new QueryWrapper<PushDetailInfo>()
                    .in("serv_id", existPushParams)
                    .ge("push_time", startDate)
                    .lt("push_time", endDate)
                );
                if (CollUtil.isNotEmpty(existList)) {
                    existList.forEach(i -> {
                        existPushMap.put(i.getServId(), i);
                    });
                }
            }
        }

        Date now = DateUtil.date();
        // 保存
        List<PushDetailInfo> saveList = new ArrayList<>();
        List<PushDetailInfo> updateList = new ArrayList<>();
        List<String> delList = new ArrayList<>(); // 删除标记为无家属的数据
        if (CollUtil.isNotEmpty(boList)) {
            boList.forEach(i -> {
                PushDetailInfo info = existMap.get(i.getRemindId());
                if (null == info) {
                    info = new PushDetailInfo();
                    info.setId(IdUtil.getSnowflake(1,1).nextIdStr());
                    saveList.add(info);
                } else {
                    updateList.add(info);
                }

                // 删除标记为无家属的数据
                if (StrUtil.isNotBlank(i.getRemindId())) {
                    PushDetailInfo del = existPushMap.get(i.getServId());
                    if (null != del && StrUtil.isBlank(del.getRemindId())) {
                        delList.add(del.getId());
                        return;
                    }
                }

                // info.setDeptCode();
                info.setPatientId(i.getPatientId());
                info.setServConfigId(i.getServConfigId());
                info.setServId(i.getServId());
                info.setRemindId(i.getRemindId());
                info.setUserId(i.getUserId());
                info.setPushTime(i.getPushTime());
                info.setPushStatus(i.getPushStatus());
                info.setReadStatus(i.getReadStatus());
                info.setReadTime(i.getReadTime());
                info.setActionStatus(i.getActionStatus());
                info.setActionTime(i.getActionTime());
                info.setStatus(i.getStatus());
                info.setLatestTime(now);
            });
        }

        if (CollUtil.isNotEmpty(saveList)) {
            pushDetailInfoService.saveBatch(saveList);
        }
        if (CollUtil.isNotEmpty(updateList)) {
            pushDetailInfoService.updateBatchById(updateList);
        }
        if (CollUtil.isNotEmpty(delList)) {
            pushDetailInfoService.removeBatchByIds(delList);
        }
    }

    @Override
    public void pushDetailUpdateStatisticsByDate(String startDate, String endDate) {
        DateTime now = DateUtil.parseDate(DateUtil.formatDate(DateUtil.date()));
        
        if (StrUtil.isBlank(startDate)) {
            startDate = DateUtil.offsetDay(now, -(maxDays + 1)).toDateStr();
        }
        DateTime start = DateUtil.parseDate(startDate);
        DateTime finishDate = DateUtil.offsetDay(now, 1);
        if (StrUtil.isNotBlank(endDate)) {
            finishDate = DateUtil.parseDate(endDate);
        }
        while (true) {
            DateTime end = DateUtil.offsetDay(start, 1);
            if (end.after(finishDate)) {
                return;
            }
            String status = YN.YES.getCode();
            if (DateUtil.offsetDay(start, maxDays).after(now)) { // 若最大数据变化日期大于当前日期则说明该数据需要后续更新
                status = YN.NO.getCode();
            }
            pushDetailUpdateStatisticsByDateHander(start.toDateStr(), end.toDateStr(), status);
            start = end;
        }
    }

    public void pushDetailUpdateStatisticsByDateHander(String startDate, String endDate, String status) {
        // 按指定配置的时间查询
        List<PushDetailInfo> list = pushDetailInfoService.list(new QueryWrapper<PushDetailInfo>()
            .eq("status", YN.NO.getCode())
            .eq("push_status", PushConstant.PushStatus.Success.getCode()) // 只有推送成功的才更新行为信息
            .ge("push_time", startDate)
            .lt("push_time", endDate)
        );

        if (CollUtil.isEmpty(list)) {
            return;
        }

        // 1.整理出未阅读/没行为的数据
        List<String> remindIds = new ArrayList<>();
        list.forEach(i -> {
            remindIds.add(i.getRemindId());
        });

        // DateTime maxDayDate = DateUtil.offsetDay(DateUtil.parseDate(startDate), maxDays);
        // 判断行为
        Map<String, PushDetailQueryBO> patientPushServeBehaviorMap = new HashMap<>();
        if (CollUtil.isNotEmpty(remindIds)) {
            List<PushDetailQueryBO> patientPushServeBehaviorList = statisticsDataQueryByBizMapper.queryPushServeBehaviorInfo(new QueryWrapper<Void>()
                .eq("source_one_type", "remind")
                .in("source_one_id", remindIds)
                // .ge("operated_start_date", startDate)
                // .lt("operated_start_date", maxDayDate)
                .groupBy("source_one_id")
            );
            if (CollUtil.isNotEmpty(patientPushServeBehaviorList)) {
                patientPushServeBehaviorList.forEach(i -> {
                    patientPushServeBehaviorMap.put(i.getRemindId(), i);
                });
            }
        }

        Date now = DateUtil.date();
        // 更新
        list.forEach(i -> {
            i.setStatus(status);
            
            PushDetailQueryBO bo = patientPushServeBehaviorMap.get(i.getRemindId());
            if (null != bo) {
                if (YN.NO.getCode().equals(i.getReadStatus()) && YN.YES.getCode().equals(bo.getReadStatus())) {
                    i.setReadStatus(YN.YES.getCode());
                    i.setReadTime(bo.getReadTime());
                }
                if (YN.NO.getCode().equals(i.getActionStatus()) && YN.YES.getCode().equals(bo.getActionStatus())) {
                    i.setActionStatus(YN.YES.getCode());
                    i.setActionTime(bo.getActionTime());
                }
                
                if (YN.YES.getCode().equals(i.getReadStatus()) && YN.YES.getCode().equals(i.getActionStatus())) {
                    i.setStatus(YN.YES.getCode());
                }
            }

            i.setLatestTime(now);
        });

        pushDetailInfoService.updateBatchById(list);
    }

    @Override
    public void pushDayStatisticsByDate(String startDate, String endDate) {
        DateTime now = DateUtil.date();
        
        if (StrUtil.isBlank(startDate)) {
            startDate = DateUtil.offsetDay(now, -maxDays).toDateStr();
        }
        DateTime start = DateUtil.parseDate(startDate);
        DateTime finishDate = DateUtil.offsetDay(now, 1);
        if (StrUtil.isNotBlank(endDate)) {
            finishDate = DateUtil.parseDate(endDate);
        }

        // 循环每天处理
        while (true) {
            if (start.after(finishDate)) {
                return;
            }
            DateTime end = DateUtil.offsetDay(start, 1);

            pushDayStatisticsByDateHander(start.toDateStr(), end.toDateStr());
            start = end;
        }
    }

    public void pushDayStatisticsByDateHander(String startDate, String endDate) {
        if (StrUtil.isBlank(startDate)) {
            return;
        }
        
        List<PushDayQueryBO> list = statisticsDataQueryByBizMapper.queryPushDayList(new QueryWrapper<Void>()
            .ge("push_time", startDate)
            .lt(StrUtil.isNotBlank(endDate), "push_time", endDate)
            .groupBy("dataTime")
        ); 
        if(CollUtil.isEmpty(list)){
            return;
        }

        Set<String> dateList = new HashSet<>();
        list.forEach(item -> {
            dateList.add(item.getDataTime());
        });

        // 查询已统计数据
        Map<String, PushDayInfo> existMap = new HashMap<>();
        List<PushDayInfo> existList = pushDayInfoService.list(new QueryWrapper<PushDayInfo>()
            .in("data_time", dateList)
        );
        if (CollUtil.isNotEmpty(existList)) {
            existList.forEach(i -> existMap.put(i.getDataTime(), i));
        }

        Date latestTime = new Date();

        List<PushDayInfo> saveList = new ArrayList<>();
        List<PushDayInfo> updateList = new ArrayList<>();

        list.forEach(item->{
            PushDayInfo entity = existMap.get(item.getDataTime());
            if (null == entity) {
                entity = new PushDayInfo();
                entity.setId(item.getDataTime());
                saveList.add(entity);
            } else {
                updateList.add(entity);
            }
            entity.setDataTime(item.getDataTime());
            // entity.setDeptCode(item.getDeptCode());
            entity.setNum(item.getNum());
            entity.setPushSuccessNum(item.getPushSuccessNum());
            entity.setPushFailUserNum(item.getPushFailUserNum());
            entity.setPushFailFollowNum(item.getPushFailFollowNum());
            entity.setPushFailNum(item.getPushFailNum());
            entity.setPushInvalidNum(item.getPushInvalidNum());
            entity.setPushTimeoutNum(item.getPushTimeoutNum());
            entity.setPushBlackListNum(item.getPushBlackListNum());
            entity.setPushAbnormalNum(item.getPushAbnormalNum());
            entity.setReadNum(item.getReadNum());
            entity.setActionNum(item.getActionNum());
            entity.setLatestTime(latestTime);

        });

        if (CollUtil.isNotEmpty(saveList)) {
            pushDayInfoService.saveBatch(saveList);
        }
        if (CollUtil.isNotEmpty(updateList)) {
            pushDayInfoService.updateBatchById(updateList);
        }
    }

}
