package com.xxh.cloud.data.statistics.framework.modules.task.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.*;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.dto.PatientDiagnosisByPatientQueryDTO;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.dto.PatientVisitRecordQueryDTO;
import com.xxh.cloud.data.statistics.framework.modules.task.mapper.StatisticsDataQueryByDwdMapper;
import com.xxh.cloud.data.statistics.framework.modules.task.service.StatisticsDwdDataQueryService;
import com.xxh.cloud.data.statistics.framework.modules.task.utils.DataStatisticsTimeUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

@DS("wd")
@Service
public class StatisticsDwdDataQueryServiceImpl implements StatisticsDwdDataQueryService {

    @Resource
    private StatisticsDataQueryByDwdMapper statisticsDataQueryByDwdMapper;


    //-----------------------------病种统计
    /**
     * 统计病种的确诊患者数量
     * */
    @Override
    public List<PatientDiagnosisDayQueryBO> getPatientDiagnosisDayList(String date) {

        Date date1= DataStatisticsTimeUtils.str2Date(date);
        String begin= DataStatisticsTimeUtils.beginOfDay(date1);
        String end= DataStatisticsTimeUtils.endOfDay(date1);

        List<PatientDiagnosisDayQueryBO> data= statisticsDataQueryByDwdMapper.queryPatientDiagnosisDay(
                new QueryWrapper<PatientDiagnosisDayQueryBO>()
                        .ge("diagnosis_time",begin)
                        .le("diagnosis_time",end)
                        .eq("sort_idx",1)
                        .groupBy("department_code_standard","scene_type_standard","diagnosis_code_standard")
        );
        return data;
    }

    //-----------------------------患者确诊列表
    /**
     * 查询:患者的诊断列表
     * */
    @Override
    public List<PatientDiagnosisByPatientQueryBO> getPatientDiagnosisList(PatientDiagnosisByPatientQueryDTO dto) {
        List<PatientDiagnosisByPatientQueryBO> data= statisticsDataQueryByDwdMapper.queryDiagnosisCodeByPatient(
                new QueryWrapper<PatientDiagnosisByPatientQueryBO>()
                        .eq(StrUtil.isNotEmpty(dto.getPatientId()),"patient_id", dto.getPatientId())
                        .in(CollUtil.isNotEmpty(dto.getPatientIds()),"patient_id", dto.getPatientIds())
                        .ge(StrUtil.isNotEmpty(dto.getStartDate()),"diagnosis_time", dto.getStartDate())
                        .le(StrUtil.isNotEmpty(dto.getEndDate()),"diagnosis_time", dto.getEndDate())
                        .eq(StrUtil.isNotEmpty(dto.getSceneCode()),"scene_type_standard", dto.getSceneCode())
                        .eq(dto.isMain(),"sort_idx", "1")
                        .groupBy("patient_id","scene_type_standard")
        );

        return data;
    }

    //-----------------------------就诊记录(复诊)
    @Override
    public List<PatientVisitRecordBO> getPatientVisitRecordList(PatientVisitRecordQueryDTO dto) {
        List<PatientVisitRecordBO> data= statisticsDataQueryByDwdMapper.selectPatientVisitReport(
                new QueryWrapper<PatientVisitRecordBO>()
                        .eq(StrUtil.isNotEmpty(dto.getPatientId()),"patient_id", dto.getPatientId())
                        .in(CollUtil.isNotEmpty(dto.getPatientIds()),"patient_id", dto.getPatientIds())
                        .ge(StrUtil.isNotEmpty(dto.getStartDate()),"visit_date", dto.getStartDate())
                        .le(StrUtil.isNotEmpty(dto.getEndDate()),"visit_date", dto.getEndDate())
                        .orderByDesc("visit_date")
        );
        return data;
    }





    //-----------------------------住院
    /**
     * 查询：开住院单患者列表
     * @param date : yyyy-mm-dd
     * */
    @Override
    public List<PatientInpatientApplyQueryBO> getPatientInpatientApplyList(String date) {
        List<PatientInpatientApplyQueryBO> inpatientApplyBOList = statisticsDataQueryByDwdMapper.selectPatientInpatientApply(new QueryWrapper<PatientInpatientApplyQueryBO>()
                .ge("apply_date", date)
                .le("apply_date", date));

        return inpatientApplyBOList;
    }


    /**
     * 查询：一段时间内住院记录
     * */
    @Override
    public List<PatientInpatientRecordQueryBO> getPatientInpatientRecordList(String startDate, String endDate) {
        List<PatientInpatientRecordQueryBO> inpatientRecordBOList = statisticsDataQueryByDwdMapper.selectPatientInpatientRecord(new QueryWrapper<PatientInpatientRecordQueryBO>()
                .ge("inpatient_date", startDate)
                .le("inpatient_date", endDate));
        return inpatientRecordBOList;
    }


    //-----------------------------首发
    /**
     * 首发诊断
     * */
    @Override
    public List<FirstDiseaseDiagnosisQueryBO> getFirstDiagnosisList(String startDate, String endDate) {
        List<FirstDiseaseDiagnosisQueryBO> data= statisticsDataQueryByDwdMapper.queryFirstDiseaseDiagnosis(new QueryWrapper<FirstDiseaseDiagnosisQueryBO>()
                .ge("diagnosis_time ", startDate)
                .le("diagnosis_time ", endDate)
                .groupBy("patient_id","diagnosis_code_standard")
                .orderByDesc("discovery_time")
        );
        return data;
    }

    @Override
    public List<VisitDayQueryBO> getVisitDayList(String startDate,String endDate) {
        
        List<VisitDayQueryBO> data= statisticsDataQueryByDwdMapper.queryVisitDay(
                new QueryWrapper<VisitDayQueryBO>()
                        .ge("visit_date",startDate)
                        .le(StrUtil.isNotBlank(endDate), "visit_date",endDate)
                        .and(i -> i.isNotNull("department_code_standard").ne("department_code_standard", ""))
                        .groupBy("dataTime", "department_code_standard")
        );
        return data;
    }

    @Override
    public List<InDayQueryBO> getInDayList(String startDate,String endDate) {
        
        List<InDayQueryBO> data= statisticsDataQueryByDwdMapper.queryInDay(
                new QueryWrapper<InDayQueryBO>()
                        .ge("inpatient_date",startDate)
                        .le(StrUtil.isNotBlank(endDate), "inpatient_date",endDate)
                        .and(i -> i.isNotNull("admit_department_code_standard").ne("admit_department_code_standard", ""))
                        .groupBy("dataTime", "admit_department_code_standard")
        );
        return data;
    }



}
