package com.xxh.cloud.data.statistics.framework.modules.task.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.InDayInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.entity.VisitDayInfo;
import com.xxh.cloud.data.statistics.framework.modules.base.service.InDayInfoService;
import com.xxh.cloud.data.statistics.framework.modules.base.service.VisitDayInfoService;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.InDayQueryBO;
import com.xxh.cloud.data.statistics.framework.modules.task.bean.bos.VisitDayQueryBO;
import com.xxh.cloud.data.statistics.framework.modules.task.service.StatisticsDwdDataQueryService;
import com.xxh.cloud.data.statistics.framework.modules.task.service.VisitAndInDayTaskService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 就诊/住院患者人数日统计
 * */
@Service
public class VisitDayTaskServiceImpl implements VisitAndInDayTaskService {

    @Autowired
    private StatisticsDwdDataQueryService statisticsDwdDataQueryService;
    @Autowired
    private VisitDayInfoService visitDayInfoService;
    @Autowired
    private InDayInfoService inDayInfoService;

    @Override
    public void visitStatisticsByDate(String startDate, String endDate) {
        DateTime now = DateUtil.date();
        DateTime start = DateUtil.parseDate(startDate);
        DateTime finishDate = now;
        if (StrUtil.isNotBlank(endDate)) {
            finishDate = DateUtil.parseDate(endDate);
        }
        while (true) {
            DateTime end = DateUtil.offsetDay(start, 1);
            if (end.after(finishDate)) {
                return;
            }
            visitStatisticsByDateHander(start.toDateStr(), end.toDateStr());
            start = end;
        }
    }

    public void visitStatisticsByDateHander(String startDate, String endDate) {
        List<VisitDayQueryBO> list= statisticsDwdDataQueryService.getVisitDayList(startDate, endDate);
        if(CollUtil.isEmpty(list)){
            return;
        }

        Set<String> dateList = new HashSet<>();
        list.forEach(item -> {
            dateList.add(item.getDataTime());
        });

        // 查询已统计数据
        Map<String, VisitDayInfo> existMap = new HashMap<>();
        List<VisitDayInfo> existList = visitDayInfoService.list(new QueryWrapper<VisitDayInfo>()
            .in("data_time", dateList)
        );
        if (CollUtil.isNotEmpty(existList)) {
            existList.forEach(i -> existMap.put(i.getDataTime() + "-" + i.getDeptCode(), i));
        }

        Date latestTime =new Date();

        List<VisitDayInfo> saveList=new ArrayList<>();
        List<VisitDayInfo> updateList=new ArrayList<>();

        list.forEach(item->{
            VisitDayInfo entity = existMap.get(item.getDataTime() + "-" + item.getDeptCode());
            if (null == entity) {
                entity = new VisitDayInfo();
                entity.setId(item.getDeptCode() + "_" + item.getDataTime());
                saveList.add(entity);
            } else {
                updateList.add(entity);
            }
            entity.setDataTime(item.getDataTime());
            entity.setDeptCode(item.getDeptCode());
            entity.setNum(item.getNum());
            entity.setLatestTime(latestTime);

        });

        if (CollUtil.isNotEmpty(saveList)) {
            visitDayInfoService.saveBatch(saveList);
        }
        if (CollUtil.isNotEmpty(updateList)) {
            visitDayInfoService.updateBatchById(updateList);
        }
    }

    @Override
    public void inStatisticsByDate(String startDate, String endDate) {
        DateTime now = DateUtil.date();
        DateTime start = DateUtil.parseDate(startDate);
        DateTime finishDate = now;
        if (StrUtil.isNotBlank(endDate)) {
            finishDate = DateUtil.parseDate(endDate);
        }
        while (true) {
            DateTime end = DateUtil.offsetDay(start, 1);
            if (end.after(finishDate)) {
                return;
            }
            inStatisticsByDateHander(start.toDateStr(), end.toDateStr());
            start = end;
        }
    }

    public void inStatisticsByDateHander(String startDate, String endDate) {
        List<InDayQueryBO> list= statisticsDwdDataQueryService.getInDayList(startDate, endDate);
        if(CollUtil.isEmpty(list)){
            return;
        }

        Set<String> dateList = new HashSet<>();
        list.forEach(item -> {
            dateList.add(item.getDataTime());
        });

        // 查询已统计数据
        Map<String, InDayInfo> existMap = new HashMap<>();
        List<InDayInfo> existList = inDayInfoService.list(new QueryWrapper<InDayInfo>()
            .in("data_time", dateList)
        );
        if (CollUtil.isNotEmpty(existList)) {
            existList.forEach(i -> existMap.put(i.getDataTime() + "-" + i.getDeptCode(), i));
        }

        Date latestTime =new Date();

        List<InDayInfo> saveList=new ArrayList<>();
        List<InDayInfo> updateList=new ArrayList<>();

        list.forEach(item->{
            InDayInfo entity = existMap.get(item.getDataTime() + "-" + item.getDeptCode());
            if (null == entity) {
                entity = new InDayInfo();
                entity.setId(item.getDeptCode() + "_" + item.getDataTime());
                saveList.add(entity);
            } else {
                updateList.add(entity);
            }
            entity.setDataTime(item.getDataTime());
            entity.setDeptCode(item.getDeptCode());
            entity.setNum(item.getNum());
            entity.setLatestTime(latestTime);

        });

        if (CollUtil.isNotEmpty(saveList)) {
            inDayInfoService.saveBatch(saveList);
        }
        if (CollUtil.isNotEmpty(updateList)) {
            inDayInfoService.updateBatchById(updateList);
        }
    }

}
