package com.xxh.cloud.data.statistics.manager.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

import com.xxh.cloud.data.statistics.framework.modules.query.service.DataQueryService;
import com.xxh.cloud.data.statistics.framework.modules.query.service.bean.bos.ConsultationDayStatBO;
import com.xxh.cloud.data.statistics.framework.modules.query.service.bean.bos.FollowUpDayStatBO;
import com.xxh.cloud.data.statistics.framework.modules.query.service.bean.bos.InDayStatBO;
import com.xxh.cloud.data.statistics.framework.modules.query.service.bean.bos.MergeDayStatBO;
import com.xxh.cloud.data.statistics.framework.modules.query.service.bean.bos.PushDayStatBO;
import com.xxh.cloud.data.statistics.framework.modules.query.service.bean.bos.VisitDayStatBO;
import com.xxh.cloud.data.statistics.framework.modules.query.service.bean.dto.StatisticsQueryDTO;
import com.xxh.cloud.data.statistics.framework.modules.query.service.bean.vo.*;
import com.xxh.cloud.data.statistics.framework.modules.task.service.DiagnosisPatientDayTaskService;
import com.xxh.cloud.framework.common.entity.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

@SaCheckLogin
@Log4j2
@RestController
@RequestMapping("/sys/data")
@Api(tags = "管理端,数据统计服务")
public class DataStatisticsQueryController {

    @Autowired
    private DataQueryService dataQueryService;

    @PostMapping("/listDiagnosisPatientDay")
    @ApiOperation(value = "确诊人数日统计", response = StatisticsQueryDTO.class)
    public Result<List<DiagnosisPatientDayBO>> listDiagnosisPatientDay(@RequestBody StatisticsQueryDTO params) {
        return new Result<List<DiagnosisPatientDayBO>>().ok(dataQueryService.listDiagnosisPatientDay(params));
    }

    @PostMapping("/listPotentialPatientNotDiagnosisDay")
    @ApiOperation(value = "潜在患者未就诊日统计", response = StatisticsQueryDTO.class)
    public Result<List<PotentialPatientNotDiagnosisDayBO>> listPotentialPatientNotDiagnosisDay(@RequestBody StatisticsQueryDTO params) {
        return new Result<List<PotentialPatientNotDiagnosisDayBO>>().ok(dataQueryService.listPotentialPatientNotDiagnosisDay(params));
    }

    @PostMapping("/listPatientFollowupDay")
    @ApiOperation(value = "复诊患者日统计", response = StatisticsQueryDTO.class)
    public Result<List<PatientFollowupDayBO>> listPatientFollowupDay(@RequestBody StatisticsQueryDTO params) {
        return new Result<List<PatientFollowupDayBO>>().ok(dataQueryService.listPatientFollowupDay(params));
    }

    @PostMapping("/listPatientInpatientLossDay")
    @ApiOperation(value = "住院流失日统计", response = StatisticsQueryDTO.class)
    public Result<List<PatientInpatientLossDayBO>> listPatientInpatientLossDay(@RequestBody StatisticsQueryDTO params) {
        return new Result<List<PatientInpatientLossDayBO>>().ok(dataQueryService.listPatientInpatientLossDay(params));
    }

    @PostMapping("/listFirstDiseaseDay")
    @ApiOperation(value = "首发病患者日统计表", response = StatisticsQueryDTO.class)
    public Result<List<FirstDiseaseDayBO>> listFirstDiseaseDay(@RequestBody StatisticsQueryDTO params) {
        return new Result<List<FirstDiseaseDayBO>>().ok(dataQueryService.listFirstDiseaseDay(params));
    }
    
    @PostMapping("/mergeStatData")
    @ApiOperation(value = "患者就诊日统计表", response = MergeDayStatBO.class)
    public Result<MergeDayStatBO> mergeStatData(@RequestBody StatisticsQueryDTO params) {
        MergeDayStatBO data = dataQueryService.mergeStatData(params);
        return new Result<MergeDayStatBO>().ok(data);
    }
    
    @PostMapping("/listVisitDay")
    @ApiOperation(value = "患者就诊日统计表", response = VisitDayStatBO.class)
    public Result<VisitDayStatBO> listVisitDay(@RequestBody StatisticsQueryDTO params) {
        VisitDayStatBO data = dataQueryService.listVisitDay(params);

        // 同比
        params.setStartDay(DateUtil.offsetMonth(DateUtil.parseDate(params.getStartDay()), -12).toDateStr());
        params.setEndDay(DateUtil.offsetMonth(DateUtil.parseDate(params.getEndDay()), -12).toDateStr());
        if (StrUtil.isNotBlank(params.getYear())) {
            params.setYear(String.valueOf(Integer.valueOf(params.getYear()) - 1));
        }
        VisitDayStatBO onYearData = dataQueryService.listVisitDay(params);
        if (null != onYearData) {
            data.setOnYearItemNum(onYearData.getItemNum());
        }

        return new Result<VisitDayStatBO>().ok(data);
    }
    
    @PostMapping("/listInDay")
    @ApiOperation(value = "患者住院日统计表", response = InDayStatBO.class)
    public Result<InDayStatBO> listInDay(@RequestBody StatisticsQueryDTO params) {
        InDayStatBO data = dataQueryService.listInDay(params);

        // 同比
        params.setStartDay(DateUtil.offsetMonth(DateUtil.parseDate(params.getStartDay()), -12).toDateStr());
        params.setEndDay(DateUtil.offsetMonth(DateUtil.parseDate(params.getEndDay()), -12).toDateStr());
        if (StrUtil.isNotBlank(params.getYear())) {
            params.setYear(String.valueOf(Integer.valueOf(params.getYear()) - 1));
        }
        InDayStatBO onYearData = dataQueryService.listInDay(params);
        if (null != onYearData) {
            data.setOnYearItemNum(onYearData.getItemNum());
        }

        return new Result<InDayStatBO>().ok(data);
    }
    
    @PostMapping("/listPushServeDay")
    @ApiOperation(value = "患者住院日统计表", response = PushDayStatBO.class)
    public Result<PushDayStatBO> listPushServeDay(@RequestBody StatisticsQueryDTO params) {
        return new Result<PushDayStatBO>().ok(dataQueryService.listPushServeDay(params));
    }
    
    @PostMapping("/listConsultationDay")
    @ApiOperation(value = "会诊日统计表", response = ConsultationDayStatBO.class)
    public Result<ConsultationDayStatBO> listConsultationDay(@RequestBody StatisticsQueryDTO params) {
        return new Result<ConsultationDayStatBO>().ok(dataQueryService.listConsultationDay(params));
    }
    
    @PostMapping("/listFollowUpDay")
    @ApiOperation(value = "随访日统计表", response = FollowUpDayStatBO.class)
    public Result<FollowUpDayStatBO> listFollowUpDay(@RequestBody StatisticsQueryDTO params) {
        return new Result<FollowUpDayStatBO>().ok(dataQueryService.listFollowUpDay(params));
    }
}
